@@ -28,7 +28,7 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-// Version 1.08d
+// Version 1.09d
 
 #ifndef __cms_H
 
@@ -41,13 +41,11 @@
 // virtually any machine.
 
 // #define USE_FLOAT        1
-// #define USE_C            1
-#define USE_ASSEMBLER    1
+#define USE_C            1
+// #define USE_ASSEMBLER    1
 
 // 3D interpolation method - Tetrahedral is slightly faster, but not always
-// is proper. So, only trilinear assembler routine is provided. For now, is
-// faster to use the trilinear (more accurated) assembler that the C-only
-// Tetrahedral implementation. Also, tetrahedral algorithm seems it was
+// is proper.  Also, tetrahedral algorithm seems it was
 // covered by a Sakamoto's patent, now expired. Code is present here only
 // for informational purposes.
 
@@ -65,7 +63,7 @@
 // like linux, SGI, VAX, FreeBSD, BeOS, etc.
 #define NON_WINDOWS  1
 
-// Uncomment this one if you are using big endian machines (only meaningfull
+// Uncomment this one if you are using big endian machines (only meaningful
 // when NON_WINDOWS is used)
 // #define USE_BIG_ENDIAN   1
 
@@ -73,6 +71,11 @@
 // "long long" type This will speedup fixed point math. (USE_C only)
 #define USE_INT64        1
 
+
+// Some machines does not have a reliable 'swab' function. Usually
+// leave commented unless the testbed diagnoses the contrary.
+// #define USE_CUSTOM_SWAB   1
+
 // In conversion 16 -> 8 bps, it should be / 257, but doesn't matter
 // anyway if I just >> 8, and the speed gain is big. This toggle forces
 // the / 257 method, at a div per component cost. Don't touch unless
@@ -86,22 +89,37 @@
 #include <math.h>
 #include <assert.h>
 
+// Metroworks CodeWarrior
+
+#if __MWERKS__ 
+#   define unlink remove
+#   if WIN32 
+#       define USE_CUSTOM_SWAB 1
+#   else
+#       define NON_WINDOWS   1
+#   endif
+#endif
+
+
 #ifdef NON_WINDOWS
 
 // Non windows environments. Also avoid indentation on includes.
 
 #undef LCMS_DLL
 
-
 #ifdef  USE_ASSEMBLER
 #  undef  USE_ASSEMBLER
 #  define USE_C               1
 #endif
 
-#if defined(__sgi__) || defined(__sgi)
+#if defined(__sgi__) || defined(__sgi) || defined(__powerpc__)
 #   define USE_BIG_ENDIAN      1
 #endif
 
+#if TARGET_CPU_PPC
+#   define USE_BIG_ENDIAN   1
+#endif
+
 #ifdef macintosh
 #   define USE_BIG_ENDIAN      1
 #endif
@@ -125,7 +143,8 @@
 
 #include <memory.h>
 #include <string.h>
-#ifdef __GNUC__
+
+#if defined(__GNUC__) || defined(__FreeBSD__)
 #   include <sys/unistd.h>
 #endif
 
@@ -135,15 +154,18 @@ typedef unsigned int DWORD, *LPDWORD;
 typedef int BOOL;
 typedef char *LPSTR;
 typedef void *LPVOID;
+typedef void* LCMSHANDLE;
 
-#ifndef QT_VERSION
-typedef void *HANDLE;
-#endif
 
 #define ZeroMemory(p,l)     memset((p),0,(l))
 #define CopyMemory(d,s,l)   memcpy((d),(s),(l))
 #define FAR
 
+#ifndef stricmp
+#   define stricmp strcasecmp
+#endif
+
+
 #ifndef FALSE
 #       define FALSE 0
 #endif
@@ -156,7 +178,7 @@ typedef void *HANDLE;
 
 #define MAX_PATH     (256)
 #define cdecl
-#pragma pack(1)
+
 
 #else
 
@@ -167,7 +189,9 @@ typedef void *HANDLE;
 #endif
 
 #include <windows.h>
-#include <pshpack1.h>
+
+typedef HANDLE LCMSHANDLE;
+
 
 #ifdef  USE_INT64
 #  ifndef LONGLONG
@@ -180,13 +204,15 @@ typedef void *HANDLE;
 
 #include "icc34.h"          // ICC header file
 
+
 // Plus some additions
 
 #define lcmsSignature            ((icSignature)           0x6c636d73L)
 #define icSigLuvKData            ((icColorSpaceSignature) 0x4C75764BL)  // 'LuvK'
 #define icSigChromaticityTag     ((icTagSignature)        0x6368726dL)  // As per Addendum 2 to Spec. ICC.1:1998-09
 #define icSigChromaticityType    ((icTagTypeSignature)    0x6368726dL)
-#define icSigHexachromeData      (0x4d434836)                           // MCH6
+#define icSigHexachromeData      ((icColorSpaceSignature) 0x4d434836L)  // MCH6
+#define icSigParametricCurveType ((icTagTypeSignature)    0x70617261L)  // para
 
 #ifdef __cplusplus
 extern "C" {
@@ -217,7 +243,7 @@ extern "C" {
 # endif
 #endif
 
-
+#ifdef  USE_ASSEMBLER
 #ifdef __BORLANDC__
 
 #      define ASM     asm
@@ -227,7 +253,7 @@ extern "C" {
 #      define ASM     __asm
 #      define RET(v)  return
 #endif
-
+#endif
 
 #ifdef _MSC_VER
 #ifndef  stricmp
@@ -258,13 +284,17 @@ extern "C" {
 
 
 #ifndef M_PI
-        #define M_PI    3.14159265358979323846
+#       define M_PI    3.14159265358979323846
+#endif
+
+#ifndef LOGE
+#       define LOGE   0.434294481   
 #endif
 
 // ********** Little cms API ***************************************************
 
-typedef HANDLE cmsHPROFILE;        // Opaque typedefs to hide internals
-typedef HANDLE cmsHTRANSFORM;
+typedef LCMSHANDLE cmsHPROFILE;        // Opaque typedefs to hide internals
+typedef LCMSHANDLE cmsHTRANSFORM;
 
 // Format of pixel is defined by one integer, using bits as follows
 //
@@ -575,11 +605,10 @@ typedef struct {
     } cmsCIExyYTRIPLE, FAR* LPcmsCIExyYTRIPLE;
 
 
-#define D50X  (0.964294)
+
+#define D50X  (0.9642)  // Following ICC spec
 #define D50Y  (1.0)
-#define D50Z  (0.825104)
-#define D50x  (0.345868)
-#define D50y  (0.357545)
+#define D50Z  (0.8249)
 
 // Does return pointers to constant structs
 LCMSAPI LPcmsCIEXYZ LCMSEXPORT cmsD50_XYZ(void);
@@ -615,16 +644,20 @@ LCMSAPI void          LCMSEXPORT cmsFloat2XYZEncoded(WORD XYZ[3], const LPcmsCIE
 LCMSAPI void          LCMSEXPORT cmsLab2LCh(LPcmsCIELCh LCh, const LPcmsCIELab Lab);
 LCMSAPI void          LCMSEXPORT cmsLCh2Lab(LPcmsCIELab Lab, const LPcmsCIELCh LCh);
 
-
 // CIELab handling
 
 LCMSAPI double        LCMSEXPORT cmsDeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2);
+LCMSAPI double        LCMSEXPORT cmsCIE94DeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2);
+LCMSAPI double        LCMSEXPORT cmsBFDdeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2);
+LCMSAPI double        LCMSEXPORT cmsCMCdeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2);
+
 LCMSAPI void          LCMSEXPORT cmsClampLab(LPcmsCIELab Lab, double amax, double amin, double bmax, double bmin);
 
 LCMSAPI BOOL          LCMSEXPORT cmsWhitePointFromTemp(int TempK, LPcmsCIExyY WhitePoint);
+
 LCMSAPI BOOL          LCMSEXPORT cmsAdaptToIlluminant(LPcmsCIEXYZ Result,
-                                                        LPcmsCIExyY SourceWhitePt,
-                                                        LPcmsCIExyY Illuminant,
+                                                        LPcmsCIEXYZ SourceWhitePt,
+                                                        LPcmsCIEXYZ Illuminant,
                                                         LPcmsCIEXYZ Value);
 
 LCMSAPI BOOL          LCMSEXPORT cmsBuildRGB2XYZtransferMatrix(LPMAT3 r,
@@ -653,10 +686,10 @@ typedef struct {
               } cmsViewingConditions, FAR* LPcmsViewingConditions;
 
 
-LCMSAPI HANDLE        LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC2);
-LCMSAPI void          LCMSEXPORT cmsCIECAM97sDone(HANDLE hModel);
-LCMSAPI void          LCMSEXPORT cmsCIECAM97sForward(HANDLE hModel, LPcmsCIEXYZ pIn, LPcmsJCh pOut);
-LCMSAPI void          LCMSEXPORT cmsCIECAM97sReverse(HANDLE hModel, LPcmsJCh pIn,    LPcmsCIEXYZ pOut);
+LCMSAPI LCMSHANDLE    LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC2);
+LCMSAPI void          LCMSEXPORT cmsCIECAM97sDone(LCMSHANDLE hModel);
+LCMSAPI void          LCMSEXPORT cmsCIECAM97sForward(LCMSHANDLE hModel, LPcmsCIEXYZ pIn, LPcmsJCh pOut);
+LCMSAPI void          LCMSEXPORT cmsCIECAM97sReverse(LCMSHANDLE hModel, LPcmsJCh pIn,    LPcmsCIEXYZ pOut);
 
 // Gamma
 
@@ -664,6 +697,7 @@ LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma);
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double Params[]);
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsAllocGamma(int nEntries);
 LCMSAPI void          LCMSEXPORT cmsFreeGamma(LPGAMMATABLE Gamma);
+LCMSAPI void          LCMSEXPORT cmsFreeGammaTriple(LPGAMMATABLE Gamma[3]);
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma);
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsJoinGamma(LPGAMMATABLE InGamma,  LPGAMMATABLE OutGamma);
 LCMSAPI LPGAMMATABLE  LCMSEXPORT cmsJoinGammaEx(LPGAMMATABLE InGamma,  LPGAMMATABLE OutGamma, int nPoints);
@@ -682,6 +716,8 @@ LCMSAPI const char*   LCMSEXPORT cmsTakeProductInfo(cmsHPROFILE hProfile);
 LCMSAPI BOOL          LCMSEXPORT cmsIsTag(cmsHPROFILE hProfile, icTagSignature sig);
 LCMSAPI int           LCMSEXPORT cmsTakeRenderingIntent(cmsHPROFILE hProfile);
 
+LCMSAPI BOOL          LCMSEXPORT cmsTakeCharTargetData(cmsHPROFILE hProfile, char** Data, size_t* len);
+                                                   
 // Translate form our notation to ICC one
 LCMSAPI icColorSpaceSignature LCMSEXPORT _cmsICCcolorSpace(int OurNotation);
 LCMSAPI                   int LCMSEXPORT _cmsChannelsOf(icColorSpaceSignature ColorSpace);
@@ -759,16 +795,12 @@ LCMSAPI void         LCMSEXPORT cmsDoTransform(cmsHTRANSFORM Transform,
                                                  LPVOID OutputBuffer,
                                                  unsigned int Size);
 
-
 LCMSAPI void         LCMSEXPORT cmsSetAlarmCodes(int r, int g, int b);
 LCMSAPI void         LCMSEXPORT cmsGetAlarmCodes(int *r, int *g, int *b);
 
-// Profiling Extensions 
+// Profile creation - New in 1.09
 
-LCMSAPI BOOL LCMSEXPORT _cmsAddTextTag(cmsHPROFILE hProfile,  icTagSignature sig, const char* Text);
-LCMSAPI BOOL LCMSEXPORT _cmsAddXYZTag(cmsHPROFILE hProfile,   icTagSignature sig, const LPcmsCIEXYZ XYZ);
-LCMSAPI BOOL LCMSEXPORT _cmsAddLUTTag(cmsHPROFILE hProfile,   icTagSignature sig, void* lut);
-LCMSAPI BOOL LCMSEXPORT _cmsSaveProfile(cmsHPROFILE hProfile, const char* FileName);
+LCMSAPI BOOL LCMSEXPORT cmsAddTag(cmsHPROFILE hProfile, icTagSignature sig, void* data);
 
 // Error handling
 
@@ -784,11 +816,42 @@ LCMSAPI int LCMSEXPORT cmsErrorAction(int nAction);
 
 void cdecl cmsSignalError(int ErrorCode, const char *ErrorText, ...);
 
+
+// LUT manipulation
+
+typedef struct _lcms_LUT_struc LUT, FAR* LPLUT; // opaque pointer
+
+LCMSAPI LPLUT LCMSEXPORT cmsAllocLUT(void);
+LCMSAPI LPLUT LCMSEXPORT cmsAllocLinearTable(LPLUT NewLUT, LPGAMMATABLE Tables[], int nTable);
+LCMSAPI LPLUT LCMSEXPORT cmsAlloc3DGrid(LPLUT Lut, int clutPoints, int inputChan, int outputChan);
+LCMSAPI LPLUT LCMSEXPORT cmsSetMatrixLUT(LPLUT Lut, LPMAT3 M);
+LCMSAPI void  LCMSEXPORT cmsFreeLUT(LPLUT Lut);
+LCMSAPI void  LCMSEXPORT cmsEvalLUT(LPLUT Lut, WORD In[], WORD Out[]);
+LCMSAPI LPLUT LCMSEXPORT cmsReadICCLut(cmsHPROFILE hProfile, icTagSignature sig);
+
+// LUT Sampling
+
+typedef int (* _cmsSAMPLER)(register WORD In[],
+                            register WORD Out[],
+                            register LPVOID Cargo);
+
+LCMSAPI int LCMSEXPORT cmsSample3DGrid(LPLUT Lut, _cmsSAMPLER Sampler, LPVOID Cargo, DWORD dwFlags);
+
 // ***************************************************************************
 // End of Little cms API From here functions are private
 // You can use them only if using static libraries, and at your own risk of
 // be stripped or changed at futures releases.
 
+// Profiling Extensions --- Would be removed from API in future revisions
+
+LCMSAPI BOOL LCMSEXPORT _cmsAddTextTag(cmsHPROFILE hProfile,  icTagSignature sig, const char* Text);
+LCMSAPI BOOL LCMSEXPORT _cmsAddXYZTag(cmsHPROFILE hProfile,   icTagSignature sig, const LPcmsCIEXYZ XYZ);
+LCMSAPI BOOL LCMSEXPORT _cmsAddLUTTag(cmsHPROFILE hProfile,   icTagSignature sig, void* lut);
+LCMSAPI BOOL LCMSEXPORT _cmsAddGammaTag(cmsHPROFILE hProfile, icTagSignature sig, LPGAMMATABLE TransferFunction);
+LCMSAPI BOOL LCMSEXPORT _cmsAddChromaticityTag(cmsHPROFILE hProfile, icTagSignature sig, LPcmsCIExyYTRIPLE Chrm);
+
+LCMSAPI BOOL LCMSEXPORT _cmsSaveProfile(cmsHPROFILE hProfile, const char* FileName);
+
 
 // Alignment handling (needed in ReadLUT16 and ReadLUT8)
 
@@ -850,6 +913,7 @@ typedef struct {                // Matrix (Fixed 15.16)
 void cdecl VEC3init(LPVEC3 r, double x, double y, double z);   // double version
 void cdecl VEC3initF(LPWVEC3 r, double x, double y, double z); // Fix32 version
 void cdecl VEC3toFix(LPWVEC3 r, LPVEC3 v);
+void cdecl VEC3fromFix(LPVEC3 r, LPWVEC3 v);
 void cdecl VEC3scaleFix(LPWORD r, LPWVEC3 Scale);
 void cdecl VEC3swap(LPVEC3 a, LPVEC3 b);
 void cdecl VEC3divK(LPVEC3 r, LPVEC3 v, double d);
@@ -864,6 +928,7 @@ void cdecl MAT3perK(LPMAT3 r, LPMAT3 v, double d);
 int  cdecl MAT3inverse(LPMAT3 a, LPMAT3 b);
 void cdecl MAT3eval(LPVEC3 r, LPMAT3 a, LPVEC3 v);
 void cdecl MAT3toFix(LPWMAT3 r, LPMAT3 v);
+void cdecl MAT3fromFix(LPMAT3 r, LPWMAT3 v);
 void cdecl MAT3evalW(LPWVEC3 r, LPWMAT3 a, LPWVEC3 v);
 BOOL cdecl MAT3isIdentity(LPWMAT3 a, double Tolerance);
 
@@ -914,9 +979,22 @@ void cdecl cmsTetrahedralInterp16(WORD Input[],
 #define LUT_HASTL2          0x0008
 #define LUT_HAS3DGRID       0x0010
 
+// New in rev 4.0 of ICC spec
+
+#define LUT_HASMATRIX2     0x0020   // Matrix for LutAToB
+#define LUT_HASMATRIX2OFS  0x0040   // Offset for LutAToB
+
+#define LUT_HASTL3         0x0080   // 'M' curves for LutAToB
+#define LUT_HASTL4         0x0100   // 'M' curves for LutBToA
+
+#define LUT_HASMATRIX3     0x0200   // Matrix for LutBToA
+#define LUT_HASMATRIX3OFS  0x0400   // Offset for LutBToA
+
+
+
 #define MAXCHANNELS  16            // Maximum number of channels
 
-typedef struct {
+struct _lcms_LUT_struc {
 
                DWORD wFlags;
                WMAT3 Matrix;                    // 15fixed16 matrix
@@ -927,9 +1005,7 @@ typedef struct {
                unsigned int OutputEntries;
                unsigned int cLutPoints;
 
-               // I'm encoding linear tables as
-               // WORD (0..ffff) values
-
+               
                LPWORD L1[MAXCHANNELS];          // First linearization
                LPWORD L2[MAXCHANNELS];          // Last linearization
 
@@ -943,31 +1019,47 @@ typedef struct {
 
                int Intent;                       // Accomplished intent
 
-               } LUT, FAR* LPLUT;
+               // New for Rev 4.0 of spec (reserved)
+
+               WMAT3 Mat3;
+               WVEC3 Ofs3;
+               LPWORD L3[MAXCHANNELS];
+               LPWORD L4[MAXCHANNELS];
+               WMAT3 Mat4;
+               WVEC3 Ofs4;
+
+               L16PARAMS L3params;
+               L16PARAMS L4params;
+
+               unsigned int L3Entries;
+               unsigned int L4Entries;
+
+               }; // LUT, FAR* LPLUT;
+
 
 
-LPLUT cdecl cmsAllocLUT(void);
-LPLUT cdecl cmsAllocLinearTable(LPLUT NewLUT, LPGAMMATABLE Tables[], int nTable);
-LPLUT cdecl cmsAlloc3DGrid(LPLUT Lut, int clutPoints, int inputChan, int outputChan);
-void  cdecl cmsFreeLUT(LPLUT Lut);
-void  cdecl cmsEvalLUT(LPLUT Lut, WORD In[], WORD Out[]);
-LPLUT cdecl cmsReadICCLut(cmsHPROFILE hProfile, icTagSignature sig);
 
 
 // Gamma
 LPGAMMATABLE cdecl cmsScaleGamma(LPGAMMATABLE Input, Fixed32 Factor);
+BOOL         cdecl _cmsSmoothEndpoints(LPWORD Table, int nEntries);
+
 
 // Sampled curves
 
 LPSAMPLEDCURVE cdecl cmsAllocSampledCurve(int nItems);
 void           cdecl cmsFreeSampledCurve(LPSAMPLEDCURVE p);
+LPSAMPLEDCURVE cdecl cmsDupSampledCurve(LPSAMPLEDCURVE p);
+
+LPSAMPLEDCURVE cdecl cmsConvertGammaToSampledCurve(LPGAMMATABLE Gamma, int nPoints);
+LPGAMMATABLE   cdecl cmsConvertSampledCurveToGamma(LPSAMPLEDCURVE Sampled, double Max);
+
 void           cdecl cmsEndpointsOfSampledCurve(LPSAMPLEDCURVE p, double* Min, double* Max);
 void           cdecl cmsClampSampledCurve(LPSAMPLEDCURVE p, double Min, double Max);
 BOOL           cdecl cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double SmoothingLambda);
 void           cdecl cmsRescaleSampledCurve(LPSAMPLEDCURVE p, double Min, double Max, int nPoints);
 
-LPSAMPLEDCURVE cdecl cmsSmoothParametricCurve(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y,
-                                                int nResultingPoints, double SmoothingLambda);
+LPSAMPLEDCURVE cdecl cmsJoinSampledCurves(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y, int nResultingPoints);
 
 // Shaper/Matrix handling
 
@@ -997,6 +1089,7 @@ LPGAMMATABLE cdecl cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig);
 BOOL         cdecl cmsReadICCMatrixRGB2XYZ(LPMAT3 r, cmsHPROFILE hProfile);
 
 // White Point & Primary chromas handling
+BOOL cdecl cmsAdaptationMatrix(LPMAT3 r, LPcmsCIEXYZ FromIll, LPcmsCIEXYZ ToIll);
 BOOL cdecl cmsAdaptMatrixToD50(LPMAT3 r, LPcmsCIExyY SourceWhitePt);
 BOOL cdecl cmsAdaptMatrixFromD50(LPMAT3 r, LPcmsCIExyY DestWhitePt);
 
@@ -1034,7 +1127,7 @@ typedef void (* _cmsCOLORCALLBACKFN)(struct _cmstransform_struct *Transform,
 
 // intermediate pass, from WORD[] to WORD[]
 
-typedef void   (* _cmsADJFN)(WORD In[], WORD Out[], LPWVEC3 a, LPWVEC3 b);
+typedef void   (* _cmsADJFN)(WORD In[], WORD Out[], LPWMAT3 m, LPWVEC3 b);
 
 typedef void   (* _cmsTRANSFN)(struct _cmstransform_struct *Transform,
                                WORD In[], WORD Out[]);
@@ -1052,10 +1145,8 @@ typedef struct _cmstransform_struct {
 
                             // Keep formats for further reference
                             DWORD InputFormat, OutputFormat;
-
-                            // Planar support
-                            DWORD StrideIn, StrideOut;
-
+                           
+                            DWORD StrideIn, StrideOut;      // Planar support
 
                             int Intent, ProofIntent;
                             int DoGamutCheck;
@@ -1063,10 +1154,9 @@ typedef struct _cmstransform_struct {
                             cmsHPROFILE InputProfile;
                             cmsHPROFILE OutputProfile;
                             cmsHPROFILE PreviewProfile;
-
-
-                            WVEC3 a1, b1;
-                            WVEC3 a2, b2;
+                        
+                            WMAT3 m1, m2;       // Matrix holding inter PCS operation
+                            WVEC3 of1, of2;     // Offset terms
 
                             _cmsCOLORCALLBACKFN xform;
 
@@ -1130,7 +1220,7 @@ int cdecl cmsChooseCnvrt(int Absolute,
                              LPcmsCIEXYZ IlluminantOut,
 
                  _cmsADJFN *fn1,
-                 LPWVEC3 wa, LPWVEC3 wb);
+                 LPWMAT3 wm, LPWVEC3 wof);
 
 
 
@@ -1148,32 +1238,25 @@ WORD cdecl Clamp_L(Fixed32 in);
 WORD cdecl Clamp_ab(Fixed32 in);
 
 
-// Sampling
-
-typedef int (* _cmsSAMPLER)(register WORD In[],
-                register WORD Out[],
-                register LPVOID Cargo);
-
-int cdecl cmsSample3DGrid(LPLUT Lut, _cmsSAMPLER Sampler, LPVOID Cargo, DWORD dwFlags);
-
 // Precalculate device link
 BOOL cdecl _cmsPrecalculateDeviceLink(_LPcmsTRANSFORM p, int PrecissionLevel);
 
-// Recompute gamut boundary
+// Compute gamut boundary
 LPLUT cdecl _cmsComputeGamutLUT(cmsHPROFILE hProfile);
 
+// Compute softproof
+LPLUT _cmsComputeSoftProofLUT(cmsHPROFILE hProfile, int nIntent);
+
 // Find a suitable prelinearization tables, matching the given transform
 void cdecl _cmsComputePrelinearizationTablesFromXFORM(cmsHTRANSFORM h, LPLUT Grid);
 
 // These are two VITAL macros, from converting between 8 and 16 bit
-// representation. For commodity, I'm always TRUNCATING values. Rounding
-// has bad side effects, as a 0xFFFF is converted to 0x10000 and a '00' is
-// returned.
+// representation. 
 
 #define RGB_8_TO_16(rgb) (WORD) ((((WORD) (rgb)) << 8)|(rgb)) // * 257
 
 #ifdef USE_257_FOR_16_TO_8_CONVERSION
-#   define RGB_16_TO_8(rgb) (BYTE) (((WORD) (rgb)) / 257)
+#   define RGB_16_TO_8(rgb) (BYTE) (((WORD) (rgb) + 128) / 257)
 #else
 #   define RGB_16_TO_8(rgb) (BYTE) (((WORD) (rgb)) >> 8)
 #endif
@@ -1185,10 +1268,7 @@ void cdecl _cmsComputePrelinearizationTablesFromXFORM(cmsHTRANSFORM h, LPLUT Gri
 }
 #endif
 
-#ifdef NON_WINDOWS
-#pragma pack()
-#else
-#include <poppack.h>
-#endif
+
+
 #endif
 
