@@ -34,6 +34,7 @@
 
 LPGAMMATABLE LCMSEXPORT cmsAllocGamma(int nEntries);
 void         LCMSEXPORT cmsFreeGamma(LPGAMMATABLE Gamma);
+void		 LCMSEXPORT cmsFreeGammaTriple(LPGAMMATABLE Gamma[3]);
 LPGAMMATABLE LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma);
 LPGAMMATABLE LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma);
 LPGAMMATABLE LCMSEXPORT cmsJoinGamma(LPGAMMATABLE InGamma, LPGAMMATABLE OutGamma);
@@ -42,6 +43,9 @@ BOOL		 LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda);
 
 
 LPGAMMATABLE cdecl cmsScaleGamma(LPGAMMATABLE Input, Fixed32 Factor);
+BOOL         cdecl _cmsSmoothEndpoints(LPWORD Table, int nPoints);
+
+
 
 
 // Sampled curves
@@ -53,24 +57,20 @@ void		   cdecl cmsClampSampledCurve(LPSAMPLEDCURVE p, double Min, double Max);
 BOOL           cdecl cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double SmoothingLambda);
 void		   cdecl cmsRescaleSampledCurve(LPSAMPLEDCURVE p, double Min, double Max, int nPoints);
 
-LPSAMPLEDCURVE cdecl cmsSmoothParametricCurve(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y,
-												int nResultingPoints, double SmoothingLambda);
+LPSAMPLEDCURVE cdecl cmsJoinSampledCurves(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y, int nResultingPoints);
 
 
 // ----------------------------------------------------------------------------------------
+
+// #define DEBUG 1
+
+#define MAX_KNOTS	4096
+typedef float vec[MAX_KNOTS+1];
+
+
 // default gamma function
 
-// Implementing gamma as a two-stages funtion.
-// I chose the cutoff between linear-exponential stage on
-//
-//     (breakin, breakout)=(0.081, 0.018)
-//
-// The linear segment minimizes the effect of sensor noise in
-// practical devices.
-
-#define PHOTOSHOP_COMPATIBILITY    1
-
-#if 1 // def PHOTOSHOP_COMPATIBLITY --- Buggy 24/5/2000
+#if 1
 
 static
 double FGamma(double R, double x)
@@ -80,34 +80,40 @@ double FGamma(double R, double x)
 
 #else
 
-static
+// Quantization becomes evident below 
+//
+//		R^x * 65535 < 1.0
+//
+// So, we apply linear tram on
+// 
+//		R^x < 1 / 65535
+//
+//		x * log(R) < log(1) - log(65535) 
+//
+//		R < (1 /exp(log(65535) / x)
+//
+//	Somehow arbitrarely, we adopt 0.018, which corresponds
+//  with gamma 2.7 This is adequate for most situations.
+//
+//  The resulting line must be:
+//
+//	f(0) = 0
+//	f(0.018) = pow(0.018, x)
+//
+//	so, 
+//	
+//	f() = pow(0.018, x) / 0.018
+
+static 
 double FGamma(double R, double x)
 {
-       double startin, endin;
-       double startout, endout;
-       double breakin, breakout;
-       double a, c;
+	   if (R < 0.018) 
+		   return (R * pow(0.018, x)) / 0.018;
 
-
-       startin = 0; endin = 1.0;
-       startout= 0; endout = 1.0;
-
-       breakin  = 0.081;
-       breakout = 0.018;
-
-       if (R >= startin && R <= breakin)
-       {
-              return (breakout-startout)*(R-startin)/(breakin-startin) + startout;
-       }
-       else
-       {
-              a = (endout-breakout)/(1-pow((breakin-startin)/(endin-startin), x));
-              c = endout - a;
-
-              return a*pow((R-startin)/(endin-startin), x)+c;
-       }
+       return pow(R, x);
 }
 
+
 #endif
 
 LPGAMMATABLE LCMSEXPORT cmsAllocGamma(int nEntries)
@@ -133,6 +139,13 @@ void LCMSEXPORT cmsFreeGamma(LPGAMMATABLE Gamma)
 
 
 
+void LCMSEXPORT cmsFreeGammaTriple(LPGAMMATABLE Gamma[3])
+{
+    cmsFreeGamma(Gamma[0]);
+    cmsFreeGamma(Gamma[1]);
+    cmsFreeGamma(Gamma[2]);
+    Gamma[0] = Gamma[1] = Gamma[2] = NULL;
+}
 
 // Build a gamma table based on gamma constant
 
@@ -158,13 +171,22 @@ LPGAMMATABLE LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma)
               return p;
        }
 
+	   if (Gamma == 1.0) {
+
+		   for (i=0; i < nEntries; i++) {
+
+			Table[i] = _cmsQuantizeVal(i, nEntries);              
+
+		   }
+		   return p;
+	   }
 
        for (i=0; i < nEntries; i++)
        {
               R   = (double) i / (nEntries-1);
               Val = FGamma(R, Gamma);
 
-              Table[i] = (WORD) floor(Val * 0xFFFFL);
+              Table[i] = (WORD) floor(Val * 65535. + .5);
        }
 
        return p;
@@ -206,35 +228,58 @@ LPGAMMATABLE LCMSEXPORT cmsJoinGamma(LPGAMMATABLE InGamma,
 
 
 
+// New method, using smoothed parametric curves. This works FAR better.
+// We want to get 
+//
+//		y = f(g^-1(x))		; f = ingamma, g = outgamma
+//
+// And this can be parametrized as
+//
+//		y = f(t)
+//		x = g(t)
+
+
 LPGAMMATABLE LCMSEXPORT cmsJoinGammaEx(LPGAMMATABLE InGamma,
-                          LPGAMMATABLE OutGamma, int nPoints)
+									   LPGAMMATABLE OutGamma, int nPoints)
 {
-       register int i;
-       L16PARAMS L16In, L16Out;
-       LPWORD InPtr, OutPtr;
-       LPGAMMATABLE p;
 
-       p = cmsAllocGamma(nPoints);
-       if (!p) return NULL;
+	LPSAMPLEDCURVE x, y, r;
+	LPGAMMATABLE res;
+	
+	x = cmsConvertGammaToSampledCurve(InGamma,  nPoints);
+	y = cmsConvertGammaToSampledCurve(OutGamma, nPoints);
+	r = cmsJoinSampledCurves(y, x, nPoints);
 
-       cmsCalcL16Params(InGamma -> nEntries, &L16In);
-       InPtr  = InGamma -> GammaTable;
+	// Does clean "hair"
+	cmsSmoothSampledCurve(r, 0.001);
+	
+	cmsFreeSampledCurve(x);
+	cmsFreeSampledCurve(y);
 
-       cmsCalcL16Params(OutGamma -> nEntries, &L16Out);
-       OutPtr = OutGamma-> GammaTable;
+	res = cmsConvertSampledCurveToGamma(r, 65535.0);
+	cmsFreeSampledCurve(r);
 
-       for (i=0; i < nPoints; i++)
-       {
-              WORD wValIn, wValOut;
-
-              wValIn  = cmsLinearInterpLUT16(_cmsQuantizeVal(i, nPoints), InPtr, &L16In);
-              wValOut = cmsReverseLinearInterpLUT16(wValIn, OutPtr, &L16Out);
-
-              p -> GammaTable[i] = wValOut;
-       }
-
-       return p;
+	return res;
 }
+
+// This does reverse a curve using smooth parametric curves. 
+
+
+/*
+LPGAMMATABLE LCMSEXPORT cmsReverseGamma2(int nResultSamples, LPGAMMATABLE InGamma)
+{
+	LPGAMMATABLE Straight = cmsBuildGamma(InGamma->nEntries, 1.0);
+	LPGAMMATABLE Result;
+	
+	Result   = cmsJoinGammaEx(Straight, InGamma, nResultSamples);
+
+	cmsFreeGamma(Straight);
+
+	return Result;
+}
+*/
+
+
 // Reverse a gamma table
 
 LPGAMMATABLE LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma)
@@ -254,15 +299,20 @@ LPGAMMATABLE LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma
        {
               WORD wValIn, wValOut;
 
-              wValIn = (WORD) ((int) ((int) i * 0xFFFFL) / (nResultSamples - 1));
-
+			  wValIn = _cmsQuantizeVal(i, nResultSamples);           
               wValOut = cmsReverseLinearInterpLUT16(wValIn, InPtr, &L16In);
               p -> GammaTable[i] = wValOut;
        }
 
+
+	   // Does eliminate "hair" on curve	  
+	   cmsSmoothGamma(p, 0.001);	   	   
+	   	   
        return p;
 }
 
+
+
 LPGAMMATABLE cmsScaleGamma(LPGAMMATABLE Input, Fixed32 Factor)
 {
        LPGAMMATABLE Output;
@@ -285,7 +335,7 @@ LPGAMMATABLE cmsScaleGamma(LPGAMMATABLE Input, Fixed32 Factor)
 LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double Params[])
 {
         LPGAMMATABLE Table;
-        double R, Val;
+        double R, Val, dval, e;
         int i;
 
         Table = cmsAllocGamma(nEntries);
@@ -308,9 +358,16 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
                 // Y = 0                  | else
 
                 case 1:
-                      if (R >= -Params[1] && R <= Params[2])
-                              Val = pow(Params[1]*R + Params[2], Params[0]) + Params[3];
-                      else
+					if (R >= -Params[1] && R <= Params[2]) {
+							  
+							  e = Params[1]*R + Params[2];
+
+							  if (e > 0)
+								Val = pow(e, Params[0]) + Params[3];
+							  else
+								Val = Params[3];
+					}
+                    else
                               Val = 0;
                       break;
 
@@ -319,9 +376,15 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
                 // Y = c                  | else
 
                 case 2:
-                      if (R >= -Params[1] && R <= Params[2])
-                              Val = pow(Params[1]*R + Params[2], Params[0]) + Params[3];
-                      else
+					if (R >= -Params[1] && R <= Params[2]) {
+							
+							  e = Params[1]*R + Params[2];
+							  if (e > 0)
+								Val = pow(e, Params[0]) + Params[3];
+							  else
+								Val = Params[3];
+					}
+                    else
                               Val = Params[3];
                       break;
 
@@ -330,8 +393,14 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
                 // Y = cX             | X < d
 
                 case 3:
-                      if (R >= Params[4])
-                              Val = pow(Params[1]*R + Params[2], Params[0]);
+					if (R >= Params[4]) {
+							  
+							  e = Params[1]*R + Params[2];
+							  if (e > 0)
+								Val = pow(e, Params[0]);
+							  else
+								Val = 0;
+					}
                       else
                               Val = R * Params[3];
                       break;
@@ -340,11 +409,16 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
                 // Y = 0                  | else
 
                 case 4:
-                      if (R >= Params[4])
-                              Val = pow(Params[1]*R + Params[2], Params[0]) + Params[3];
-                      else
+					if (R >= Params[4]) {
+							  e = Params[1]*R + Params[2];
+							  if (e > 0) 
+								Val = pow(e, Params[0]) + Params[3];
+							  else
+								Val = Params[3];
+					}
+                    else
                               Val = R*Params[5] + Params[6];
-                      break;
+                    break;
 
                 default:
                         cmsSignalError(-1, "Unsupported parametrized curve");
@@ -353,7 +427,13 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
                 }
 
 
-        Table->GammaTable[i] = (WORD) floor(Val * 0xFFFFL);
+		// Saturate
+
+		dval = Val * 65535.0 + .5;
+		if (dval > 65535.) dval = 65535.0;
+		if (dval < 0) dval = 0;
+
+        Table->GammaTable[i] = (WORD) floor(dval);
         }
 
         return Table;
@@ -371,9 +451,6 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
 //   Output: smoothed vector (z): vector from 1 to m.
 
 
-#define MAX_KNOTS	4096
-typedef float vec[MAX_KNOTS+1];
-
 static
 void smooth2(vec w, vec y, vec z, float lambda, int m)
 {
@@ -461,7 +538,7 @@ BOOL LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda)
 		if (v < 0) v = 0;
 		if (v > 65535.) v = 65535.;
 
-		Tab -> GammaTable[i] = (WORD) floor(v);
+		Tab -> GammaTable[i] = (WORD) floor(v + .5);
         }
 
 	return TRUE;
@@ -478,13 +555,16 @@ double LCMSEXPORT cmsEstimateGamma(LPGAMMATABLE t)
 
     sum = sum2 = n = 0;
     
-    // Does exclude endpoints
+    // Does exclude endpoints	
     for (i=1; i < t ->nEntries - 1; i++) {
 
             x = (double) i / (t -> nEntries - 1);
             y = (double) t -> GammaTable[i] / 65535.;
-                    
-            if (y > 0. && y < 1.) {
+            
+			// Avoid 7% on lower part to prevent 
+			// artifacts due to linear ramps
+
+            if (y > 0. && y < 1. && x > 0.07) {
 
             gamma = log(y) / log(x);
             sum  += gamma;
@@ -538,6 +618,22 @@ void cmsFreeSampledCurve(LPSAMPLEDCURVE p)
 }
 
 
+
+// Does duplicate a sampled curve
+
+LPSAMPLEDCURVE cmsDupSampledCurve(LPSAMPLEDCURVE p)
+{
+	LPSAMPLEDCURVE out;
+
+	out = cmsAllocSampledCurve(p -> nItems);
+	if (!out) return NULL;
+
+	CopyMemory(out ->Values, p ->Values, p->nItems * sizeof(double));
+
+	return out;
+}
+
+
 // Take min, max of curve
 
 void cmsEndpointsOfSampledCurve(LPSAMPLEDCURVE p, double* Min, double* Max)
@@ -550,12 +646,16 @@ void cmsEndpointsOfSampledCurve(LPSAMPLEDCURVE p, double* Min, double* Max)
         for (i=0; i < p -> nItems; i++) {
 
                 double v = p -> Values[i];
+
                 if (v < *Min)
                         *Min = v;
 
                 if (v > *Max)
                         *Max = v;
         }
+
+		if (*Min < 0) *Min = 0;
+		if (*Max > 65535.0) *Max = 65535.0;
 }
 
 // Clamps to Min, Max
@@ -582,6 +682,7 @@ void cmsClampSampledCurve(LPSAMPLEDCURVE p, double Min, double Max)
 }
 
 
+
 // Smooths a curve sampled at regular intervals
 
 BOOL cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double lambda)
@@ -602,10 +703,13 @@ BOOL cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double lambda)
 
 	for (i=0; i < nItems; i++)
 	{
-		y[i+1] = (float) Tab -> Values[i];
-		w[i+1] = 1.0;
+		float value = (float) Tab -> Values[i];
+
+		y[i+1] = value;
+		w[i+1] = (float) ((value < 0.0) ?  0 : 1);
 	}
 
+
 	smooth2(w, y, z, (float) lambda, nItems);
 
 	for (i=0; i < nItems; i++) {
@@ -627,6 +731,8 @@ double ScaleVal(double v, double Min, double Max, int nPoints)
 
         double a, b;
 
+		if (v < 0) return 0;
+	
         a = (double) (nPoints - 1) / (Max - Min);
         b = a * Min;
 
@@ -651,16 +757,15 @@ void cmsRescaleSampledCurve(LPSAMPLEDCURVE p, double Min, double Max, int nPoint
 
 }
 
-// Smooths a parametric curve giving a sampled one. X and Y *MUST* be monotonic
 
-LPSAMPLEDCURVE cmsSmoothParametricCurve(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y,
-												int nResultingPoints, double SmoothingLambda)
+// Joins two sampled curves for X and Y
+
+LPSAMPLEDCURVE cmsJoinSampledCurves(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y, int nResultingPoints)
 {
-
-	vec w, y, z;
-	int i, pos;
+	int i, j;	
 	LPSAMPLEDCURVE out;
     double MinX, MinY, MaxX, MaxY;
+	double x, y, x1, y1, x2, y2, a, b;
 
 	out = cmsAllocSampledCurve(nResultingPoints);
     if (out == NULL)
@@ -668,34 +773,182 @@ LPSAMPLEDCURVE cmsSmoothParametricCurve(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y,
 
     if (X -> nItems != Y -> nItems) {
 
-		cmsSignalError(LCMS_ERRC_ABORTED, "cmsSmoothParametricCurve: invalid curve.");
+		cmsSignalError(LCMS_ERRC_ABORTED, "cmsJoinSampledCurves: invalid curve.");
 		cmsFreeSampledCurve(out);
 		return NULL;
     }
 
-	ZeroMemory(w, nResultingPoints * sizeof(float));
-	ZeroMemory(y, nResultingPoints * sizeof(float));
-	ZeroMemory(z, nResultingPoints * sizeof(float));
-
+	// Get endpoints of sampled curves
     cmsEndpointsOfSampledCurve(X, &MinX, &MaxX);
     cmsEndpointsOfSampledCurve(Y, &MinY, &MaxY);
 
-	// Set our points, let smooth2 to interpolate missing ones.
-	for (i=0; i < X -> nItems; i++) {
+	
+	// Set our points
+	out ->Values[0] = MinY;	
+	for (i=1; i < nResultingPoints; i++) {
 
-        pos      = (int) floor(ScaleVal(X -> Values[i], MinX, MaxX, nResultingPoints));
-		y[pos+1] = (float) Y -> Values[i]; 
-		w[pos+1] = 1.0;
+		// Scale t to x domain
+		x = (i * (MaxX - MinX) / (nResultingPoints-1)) + MinX;
+
+		// Find interval in which t is within (always up, 
+		// since fn should be monotonic at all)
+
+		j = 1;
+		while ((j < X ->nItems - 1) && X ->Values[j] < x)
+			j++;
+			
+		// Now x is within X[j-1], X[j]
+		x1 = X ->Values[j-1]; x2 = X ->Values[j];
+		y1 = Y ->Values[j-1]; y2 = Y ->Values[j];
+
+		// Interpolate  the value
+		a = (y1 - y2) / (x1 - x2);
+		b = y1 - a * x1;
+		y = a* x + b;
+		
+		out ->Values[i] = y;
 	}
+	
 
-	smooth2(w, y, z, (float) SmoothingLambda, nResultingPoints);
-
-	for (i=0; i < nResultingPoints; i++) {
-
-                double v =z[i+1];
-				out -> Values[i] = v;
-    }
-
-        
+	cmsClampSampledCurve(out, MinY, MaxY);
 	return out;
 }
+
+
+
+
+
+LPGAMMATABLE cmsConvertSampledCurveToGamma(LPSAMPLEDCURVE Sampled, double Max)
+{
+	LPGAMMATABLE Gamma;
+	int i, nPoints;
+	
+
+// 	cmsEndpointsOfSampledCurve(Sampled, &Min, &Max);
+
+	nPoints = Sampled ->nItems;
+
+	Gamma = cmsAllocGamma(nPoints);
+	for (i=0; i < nPoints; i++) {
+		
+		Gamma->GammaTable[i] = (WORD) floor(ScaleVal(Sampled ->Values[i], 0, Max, 65536) + .5);
+	}
+
+	return Gamma;
+
+}
+
+LPSAMPLEDCURVE cmsConvertGammaToSampledCurve(LPGAMMATABLE Gamma, int nPoints)
+{
+	LPSAMPLEDCURVE Sampled;
+	L16PARAMS L16;
+	int i;
+	WORD wQuant, wValIn;
+
+	if (nPoints > 4096) {
+
+		cmsSignalError(-1, "cmsConvertGammaToSampledCurve: too many points (max=4096)");
+		return NULL;
+	}
+
+	cmsCalcL16Params(Gamma -> nEntries, &L16);
+       
+	Sampled = cmsAllocSampledCurve(nPoints);
+	for (i=0; i < nPoints; i++) {
+			wQuant  = _cmsQuantizeVal(i, nPoints);
+			wValIn  = cmsLinearInterpLUT16(wQuant, Gamma ->GammaTable, &L16);
+			Sampled ->Values[i] = (float) wValIn;
+	}
+
+	return Sampled;
+}
+
+
+#ifdef DEBUG
+static
+void ASAVE(LPWORD Table, int nEntries, const char* dump)
+{
+	FILE* f;
+	int i;
+
+        f = fopen(dump, "wt");
+        if (!f)
+                return;
+
+        
+
+	for (i=0; i < nEntries; i++)
+		fprintf(f, "%g\n", (double) Table[i]);
+        
+
+	fclose(f);
+}
+#endif
+
+
+// Smooth endpoints (used in Black/White compensation)
+
+BOOL _cmsSmoothEndpoints(LPWORD Table, int nEntries)
+{
+	vec w, y, z;
+	int i, Zeros, Poles;
+
+#ifdef DEBUG
+		ASAVE(Table, nEntries, "nonsmt.txt");
+#endif
+
+
+	if (cmsIsLinear(Table, nEntries)) return FALSE; // Nothing to do
+
+    
+    if (nEntries > MAX_KNOTS) {
+                cmsSignalError(LCMS_ERRC_ABORTED, "_cmsSmoothEndpoints: too many points.");
+				return FALSE;
+                }
+
+	ZeroMemory(w, nEntries * sizeof(float));
+	ZeroMemory(y, nEntries * sizeof(float));
+	ZeroMemory(z, nEntries * sizeof(float));
+
+	for (i=0; i < nEntries; i++)
+	{
+		y[i+1] = (float) Table[i];
+		w[i+1] = 1.0;
+	}
+
+	w[1]      = 65535.0;
+	w[nEntries] = 65535.0;
+
+	smooth2(w, y, z, (float) nEntries, nEntries);
+
+	// Do some reality - checking...
+	Zeros = Poles = 0;
+	for (i=nEntries; i > 1; --i) {
+
+			if (z[i] == 0.) Zeros++;
+			if (z[i] >= 65535.) Poles++;
+			if (z[i] < z[i-1]) return FALSE; // Non-Monotonic
+	}
+
+	if (Zeros > (nEntries / 3)) return FALSE;  // Degenerated, mostly zeros
+	if (Poles > (nEntries / 3)) return FALSE;	 // Degenerated, mostly poles
+
+	// Seems ok
+
+	for (i=0; i < nEntries; i++) {
+
+		// Clamp to WORD
+
+		float v = z[i+1];
+
+		if (v < 0) v = 0;
+		if (v > 65535.) v = 65535.;
+
+		Table[i] = (WORD) floor(v + .5);
+        }
+
+#ifdef DEBUG
+		ASAVE(Table, nEntries, "smoothed.txt");
+#endif
+	return TRUE;
+}
\ No newline at end of file
