@@ -33,13 +33,13 @@
 // #define DEBUG 1
 
 /*
-Gamut check by default is an catching of 0xFFFF/0xFFFF/0xFFFF PCS values, used
+Gamut check by default is a catching of 0xFFFF/0xFFFF/0xFFFF PCS values, used
 internally by lcms to hold invalid values. Matrix LUT's, operates in a way that
 unencodeable values are marked as this combination, if PCS is XYZ, this is a very
 high value since encoding is a 1.15 fixed point, something like 1.9997, 1.9997, 1.9997
-a not very common color after all. Lab PCS is not to be a problem, since L>100 are truely
+not a very common color after all. Lab PCS is not to be a problem, since L>100 are truely
 undefined. There is a posibility than ICC comitee defines L>100 as a valid means
-to use highlights, but then, it also will be out of gamut such values.
+to use highlights, then it will be lost.
 */
 
 
@@ -146,6 +146,7 @@ double LCMSEXPORT cmsDeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2)
         if (Lab2 -> a < -200 || Lab2 -> a > 200) return 65536.;
         if (Lab2 -> b < -200 || Lab2 -> b > 200) return 65536.;
 
+		if (Lab1 ->L == 0 && Lab2 ->L == 0) return 0;
 		
         dL = fabs(Lab1 -> L - Lab2 -> L);
         da = fabs(Lab1 -> a - Lab2 -> a);
@@ -155,6 +156,160 @@ double LCMSEXPORT cmsDeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2)
 
 }
 
+
+// Square
+static
+double Sqr(double v)
+{
+	return v *  v; 
+}
+
+// Return the CIE94 Delta E 
+double LCMSEXPORT cmsCIE94DeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2)
+{
+	cmsCIELCh LCh1, LCh2;
+	double dE, dL, dC, dh, dhsq;
+	double c12, sc, sh;
+
+	if (Lab1 ->L == 0 && Lab2 ->L == 0) return 0;
+
+	dL = fabs(Lab1 ->L - Lab2 ->L);
+
+	cmsLab2LCh(&LCh1, Lab1);
+	cmsLab2LCh(&LCh2, Lab2);
+
+	dC  = fabs(LCh1.C - LCh2.C);
+	dE  = cmsDeltaE(Lab1, Lab2);
+	
+	dhsq = Sqr(dE) - Sqr(dL) - Sqr(dC);
+	if (dhsq < 0)
+		dh = 0;
+	else
+		dh = pow(dhsq, 0.5);
+
+	c12 = sqrt(LCh1.C * LCh2.C);
+
+	sc = 1.0 + (0.048 * c12);
+	sh = 1.0 + (0.014 * c12);
+			
+	return sqrt(Sqr(dL)  + Sqr(dC) / Sqr(sc) + Sqr(dh) / Sqr(sh));
+}
+
+
+// Auxiliary
+
+static
+double ComputeLBFD(LPcmsCIELab Lab)
+{
+  double yt;
+
+
+
+  if (Lab->L > 7.996969)
+		yt = (Sqr((Lab->L+16)/116)*((Lab->L+16)/116))*100;
+  else
+		yt = 100 * (Lab->L / 903.3);
+
+  return (54.6 * (LOGE * (log(yt + 1.5))) - 9.6);
+}
+
+
+
+// bfd - gets BFD(1:1) difference between Lab1, Lab2
+double LCMSEXPORT cmsBFDdeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2)
+{
+  double lbfd1,lbfd2,AveC,Aveh,dE,deltaL,
+  deltaC,deltah,dc,t,g,dh,rh,rc,rt,bfd;
+  cmsCIELCh LCh1, LCh2;
+
+
+  if (Lab1 ->L == 0 && Lab2 ->L == 0) return 0;
+
+  lbfd1 = ComputeLBFD(Lab1);
+  lbfd2 = ComputeLBFD(Lab2);
+  deltaL = lbfd2 - lbfd1;
+
+  cmsLab2LCh(&LCh1, Lab1);
+  cmsLab2LCh(&LCh2, Lab2);
+
+  deltaC = LCh2.C - LCh1.C;
+  AveC = (LCh1.C+LCh2.C)/2;
+  Aveh = (LCh1.h+LCh2.h)/2;
+
+  dE = cmsDeltaE(Lab1, Lab2);
+
+  if (Sqr(dE)>(Sqr(Lab2->L-Lab1->L)+Sqr(deltaC)))
+    deltah = sqrt(Sqr(dE)-Sqr(Lab2->L-Lab1->L)-Sqr(deltaC));
+  else
+    deltah =0;
+
+  
+  dc   = 0.035 * AveC / (1 + 0.00365 * AveC)+0.521;
+  g    = sqrt(Sqr(Sqr(AveC))/(Sqr(Sqr(AveC))+14000));
+  t    = 0.627+(0.055*cos((Aveh-254)/(180/M_PI))-       
+		 0.040*cos((2*Aveh-136)/(180/M_PI))+
+		 0.070*cos((3*Aveh-31)/(180/M_PI))+
+		 0.049*cos((4*Aveh+114)/(180/M_PI))-
+		 0.015*cos((5*Aveh-103)/(180/M_PI)));
+
+  dh    = dc*(g*t+1-g);
+  rh    = -0.260*cos((Aveh-308)/(180/M_PI))-
+         0.379*cos((2*Aveh-160)/(180/M_PI))-
+         0.636*cos((3*Aveh+254)/(180/M_PI))+
+         0.226*cos((4*Aveh+140)/(180/M_PI))-
+         0.194*cos((5*Aveh+280)/(180/M_PI));
+  
+	rc = sqrt((AveC*AveC*AveC*AveC*AveC*AveC)/((AveC*AveC*AveC*AveC*AveC*AveC)+70000000));
+	rt = rh*rc;
+  
+	bfd = sqrt(Sqr(deltaL)+Sqr(deltaC/dc)+Sqr(deltah/dh)+(rt*(deltaC/dc)*(deltah/dh)));
+
+	return bfd;
+}
+
+
+//  cmc - CMC(1:1) difference between Lab1, Lab2
+double LCMSEXPORT cmsCMCdeltaE(LPcmsCIELab Lab1, LPcmsCIELab Lab2)
+{
+  double dE,dL,dC,dh,sl,sc,sh,t,f,cmc;
+  cmsCIELCh LCh1, LCh2;
+
+  if (Lab1 ->L == 0 && Lab2 ->L == 0) return 0;
+
+  cmsLab2LCh(&LCh1, Lab1);
+  cmsLab2LCh(&LCh2, Lab2);
+
+  
+  dL = Lab2->L-Lab1->L;
+  dC = LCh2.C-LCh1.C;
+
+  dE = cmsDeltaE(Lab1, Lab2);
+  if (Sqr(dE)>(Sqr(dL)+Sqr(dC))) 
+			dh = sqrt(Sqr(dE)-Sqr(dL)-Sqr(dC));
+  else
+			dh =0;
+
+  if ((LCh1.h > 164) && (LCh1.h<345)) 
+	  t = 0.56 + fabs(0.2 * cos(((LCh1.h + 168)/(180/M_PI))));
+  else 
+	  t = 0.36 + fabs(0.4 * cos(((LCh1.h + 35 )/(180/M_PI))));
+
+   sc  = 0.0638   * LCh1.C / (1 + 0.0131  * LCh1.C) + 0.638;
+   sl  = 0.040975 * Lab1->L /(1 + 0.01765 * Lab1->L);
+   
+   if (Lab1->L<16)
+		 sl = 0.511; 
+
+   f   = sqrt((LCh1.C * LCh1.C * LCh1.C * LCh1.C)/((LCh1.C * LCh1.C * LCh1.C * LCh1.C)+1900));
+   sh  = sc*(t*f+1-f);
+   cmc = sqrt(Sqr(dL/sl)+Sqr(dC/sc)+Sqr(dh/sh));
+
+   return cmc;
+}
+
+   
+	
+
 // Carefully,  clamp on CIELab space.
 
 void LCMSEXPORT cmsClampLab(LPcmsCIELab Lab, double amax, double amin,
@@ -314,9 +469,7 @@ int LCMSEXPORT _cmsChannelsOf(icColorSpaceSignature ColorSpace)
 
 
 
-// This sampler does compute gamut boundaries by comparing original
-// values with a transform going back and forth. There is a thereshold
-// to safeguard against roundoff errors
+// Used by gamut & softproofing
 
 typedef struct {
 
@@ -324,6 +477,9 @@ typedef struct {
 
 	} GAMUTCHAIN,FAR* LPGAMUTCHAIN;
 
+// This sampler does compute gamut boundaries by comparing original
+// values with a transform going back and forth. There is a thereshold
+// to safeguard against roundoff errors
 
 static
 int GamutSampler(register WORD In[], register WORD Out[], register LPVOID Cargo)
@@ -369,6 +525,10 @@ int GamutSampler(register WORD In[], register WORD Out[], register LPVOID Cargo)
 }
 
 
+
+
+
+
 // Does compute a gamut LUT going back and forth across 
 // pcs -> relativ. colorimetric intent -> pcs
 // the dE obtained is then annotated on the LUT.
@@ -396,8 +556,7 @@ LPLUT _cmsComputeGamutLUT(cmsHPROFILE hProfile)
 
 	hLab = cmsCreateLabProfile(NULL);
 
-	// ONLY 4 channels, because this is the
-	// current limit in 3d interpolation.
+	// ONLY 4 channels	
 	dwFormat = (CHANNELS_SH(4)|BYTES_SH(2));
 	
 	// Safeguard against early abortion
@@ -447,6 +606,86 @@ LPLUT _cmsComputeGamutLUT(cmsHPROFILE hProfile)
 	return Gamut;
 }
 
+// SoftProofing. Convert from Lab to device, then back to Lab, 
+// any gamut remapping is applied
+
+static
+int SoftProofSampler(register WORD In[], register WORD Out[], register LPVOID Cargo)
+{
+		LPGAMUTCHAIN t = (LPGAMUTCHAIN) Cargo;
+		WORD Colorant[MAXCHANNELS];
+				
+		// From pcs to colorant
+        cmsDoTransform(t -> hForward, In, Colorant, 1);
+
+		// Now, do the inverse, from colorant to pcs.
+		cmsDoTransform(t -> hReverse, Colorant, Out, 1);
+				
+		return TRUE;
+}
+
+// Does return Softproofing LUT on desired intent
+
+LPLUT _cmsComputeSoftProofLUT(cmsHPROFILE hProfile, int nIntent)
+{
+	cmsHPROFILE hLab;
+	LPLUT SoftProof;
+	DWORD dwFormat;
+	GAMUTCHAIN Chain;
+	int nErrState;
+		
+
+	ZeroMemory(&Chain, sizeof(GAMUTCHAIN));
+
+	hLab = cmsCreateLabProfile(NULL);
+
+	// ONLY 4 channels	
+	dwFormat = (CHANNELS_SH(4)|BYTES_SH(2));
+	
+	// Safeguard against early abortion
+	nErrState = cmsErrorAction(LCMS_ERROR_IGNORE);
+
+	// Does create the first step
+	Chain.hForward = cmsCreateTransform(hLab, TYPE_Lab_16, 
+										hProfile, dwFormat, 
+										nIntent,
+										cmsFLAGS_NOTPRECALC);
+
+	// Does create the last step
+	Chain.hReverse = cmsCreateTransform(hProfile, dwFormat, 
+										hLab, TYPE_Lab_16, 										
+										nIntent, 
+										cmsFLAGS_NOTPRECALC);
+
+	// Restores error handler previous state
+	cmsErrorAction(nErrState);
+
+	// All ok?
+	if (Chain.hForward && Chain.hReverse) {
+				
+	// This is Lab -> Lab, so 33 point should hold anything
+	SoftProof = cmsAllocLUT();
+	SoftProof = cmsAlloc3DGrid(SoftProof, 33, 3, 3);
+
+	cmsSample3DGrid(SoftProof, SoftProofSampler, (LPVOID) &Chain, 0);
+	}
+	else 
+		SoftProof = NULL;	// Didn't work...
+
+	// Free all needed stuff.
+	if (Chain.hForward) cmsDeleteTransform(Chain.hForward);
+	if (Chain.hReverse) cmsDeleteTransform(Chain.hReverse);
+
+	cmsCloseProfile(hLab);
+
+	return SoftProof;
+}
+
+
+
+
+
+
 #ifdef DEBUG
 static
 void ASAVE(LPGAMMATABLE p, const char* dump)
