@@ -1,6 +1,6 @@
 //
 //  Little cms
-//  Copyright (C) 1998-2000 Marti Maria
+//  Copyright (C) 1998-2002 Marti Maria
 //
 // THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 // EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
@@ -27,6 +27,8 @@
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+// 
+// 1.09b
 
 
 //     Input/Output
@@ -44,7 +46,7 @@ cmsHPROFILE   LCMSEXPORT cmsCreateRGBProfile(LPcmsCIExyY WhitePoint,
                                         LPGAMMATABLE TransferFunction[3]);
 
 BOOL          LCMSEXPORT cmsCloseProfile(cmsHPROFILE hProfile);
-LPLUT         cdecl      cmsReadICCLut(cmsHPROFILE hProfile, icTagSignature sig);
+LPLUT         LCMSEXPORT cmsReadICCLut(cmsHPROFILE hProfile, icTagSignature sig);
 LPGAMMATABLE  cdecl      cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig);
 BOOL          LCMSEXPORT cmsTakeMediaWhitePoint(LPcmsCIEXYZ Dest, cmsHPROFILE hProfile);
 BOOL          LCMSEXPORT cmsTakeMediaBlackPoint(LPcmsCIEXYZ Dest, cmsHPROFILE hProfile);
@@ -61,9 +63,20 @@ icProfileClassSignature LCMSEXPORT cmsGetDeviceClass(cmsHPROFILE hProfile);
 
 int                     cdecl cmsIsLinear(WORD Table[], int nEntries);
 
+BOOL LCMSEXPORT _cmsAddLUTTag(cmsHPROFILE hProfile, icTagSignature sig, void* lut);
+BOOL LCMSEXPORT _cmsAddGammaTag(cmsHPROFILE hProfile, icTagSignature sig, LPGAMMATABLE TransferFunction);
+BOOL LCMSEXPORT _cmsAddChromaticityTag(cmsHPROFILE hProfile, icTagSignature sig, LPcmsCIExyYTRIPLE Chrm);
+
+
+
+
+BOOL LCMSEXPORT cmsAddTag(cmsHPROFILE hProfile, icTagSignature sig, void* Tag);
+BOOL LCMSEXPORT _cmsSaveProfile(cmsHPROFILE hProfile, const char* FileName);
 
 // ------------------- implementation -----------------------------------
 
+
+
 #define MAX_TABLE_TAG       50
 
 #define ALIGNLONG(x) (((x)+3) & ~(3))         // Aligns to DWORD boundary
@@ -98,11 +111,18 @@ typedef struct {
 
                char   PhysicalFile[MAX_PATH];
                BOOL   IsTemporary;
+			   BOOL   IsWrite;
 
               } ICCPROFILE, FAR* LPICCPROFILE;
 
 
+
 #ifdef __BEOS__
+#		define USE_CUSTOM_SWAB	1
+#endif
+
+
+#ifdef USE_CUSTOM_SWAB
 
 // Replacement to swab function, thanks to YNOP
 // for providing the BeOS port
@@ -110,7 +130,7 @@ typedef struct {
 // from: @(#)swab.c       5.10 (Berkeley)  3/6/91
 
 static
-void swab(const void *from, void *to, size_t len)
+void xswab(const void *from, void *to, size_t len)
 {
          register unsigned long temp;
          register int n;
@@ -131,6 +151,8 @@ void swab(const void *from, void *to, size_t len)
          }
 #undef STEP
 }
+#else
+#define xswab swab
 #endif
 
 
@@ -174,7 +196,7 @@ void AdjustEndianess32(LPBYTE pByte)
 static
 void AdjustEndianessArray16(LPWORD p, size_t num_words)
 {
-       swab((char*) p, (char*)p, (int) num_words * sizeof(WORD));
+       xswab((char*) p, (char*)p, (int) num_words * sizeof(WORD));
 }
 
 #endif
@@ -216,6 +238,19 @@ double Convert15Fixed16(icS15Fixed16Number fix32)
 }
 
 
+// Read checking result
+
+static
+size_t SafeRead(void *buffer, size_t size, size_t count, FILE *stream)
+{
+	size_t nReaded = fread(buffer, size, count, stream);
+	if (nReaded != count) {
+			cmsSignalError(LCMS_ERRC_WARNING, "Read error. Got %d entries, table should be of %d entries", nReaded, count);			
+	}
+
+	return nReaded;
+}
+
 
 static
 ICCPROFILE *ICCFileOpen(FILE *ICCfile)
@@ -225,7 +260,7 @@ ICCPROFILE *ICCFileOpen(FILE *ICCfile)
        icHeader Header;
        icInt32Number TagCount, i;
 
-       fread(&Header, sizeof(icHeader), 1, ICCfile);
+       SafeRead(&Header, sizeof(icHeader), 1, ICCfile);
 
        // Convert endian
 
@@ -259,7 +294,7 @@ ICCPROFILE *ICCFileOpen(FILE *ICCfile)
 
        Icc -> stream = ICCfile;
 
-       if (fread(&TagCount, sizeof(icInt32Number), 1, ICCfile) != 1)
+       if (SafeRead(&TagCount, sizeof(icInt32Number), 1, ICCfile) != 1)
                      goto ErrorCleanup;
 
        AdjustEndianess32((LPBYTE) &TagCount);
@@ -278,7 +313,7 @@ ICCPROFILE *ICCFileOpen(FILE *ICCfile)
        Icc -> TagCount   = TagCount;
        for (i=0; i < TagCount; i++)
        {
-              fread(&Tag, sizeof(icTag), 1, ICCfile);
+              SafeRead(&Tag, sizeof(icTag), 1, ICCfile);
 
               AdjustEndianess32((LPBYTE) &Tag.offset);
               AdjustEndianess32((LPBYTE) &Tag.size);
@@ -382,6 +417,8 @@ cmsHPROFILE LCMSEXPORT cmsCreateRGBProfile(LPcmsCIExyY WhitePoint,
        cmsCIEXYZ tmp;
        MAT3 MColorants;
        cmsCIEXYZTRIPLE Colorants;
+	   cmsCIExyY MaxWhite;
+	  
 
        Icc = (ICCPROFILE *) malloc(sizeof(ICCPROFILE));
        if (!Icc)                          // can't allocate
@@ -397,9 +434,9 @@ cmsHPROFILE LCMSEXPORT cmsCreateRGBProfile(LPcmsCIExyY WhitePoint,
        Icc -> PCS = icSigXYZData;
        Icc -> RenderingIntent = icPerceptual;    // icRelativeColorimetric ?
        Icc -> flags = 0;
-       Icc -> Illuminant.X    = 0.964294;
-       Icc -> Illuminant.Y    = 1.0;
-       Icc -> Illuminant.Z    = 0.825104;
+       Icc -> Illuminant.X    = D50X;
+       Icc -> Illuminant.Y    = D50Y;
+       Icc -> Illuminant.Z    = D50Z;
 
 
 
@@ -435,16 +472,20 @@ cmsHPROFILE LCMSEXPORT cmsCreateRGBProfile(LPcmsCIExyY WhitePoint,
 
        if (WhitePoint && Primaries) {
 
-       if (!cmsBuildRGB2XYZtransferMatrix(&MColorants, WhitePoint, Primaries))
+		MaxWhite.x =  WhitePoint -> x;
+		MaxWhite.y =  WhitePoint -> y;
+		MaxWhite.Y =  1.0;
+
+       if (!cmsBuildRGB2XYZtransferMatrix(&MColorants, &MaxWhite, Primaries))
        {
               free(Icc);
-              return FALSE;
+              return NULL;
        }  
 
 
-	   
+	 
 
-	   cmsAdaptMatrixToD50(&MColorants, WhitePoint);
+	   cmsAdaptMatrixToD50(&MColorants, &MaxWhite);
 
        Colorants.Red.X = MColorants.v[0].n[0];
        Colorants.Red.Y = MColorants.v[1].n[0];
@@ -481,33 +522,29 @@ cmsHPROFILE LCMSEXPORT cmsCreateRGBProfile(LPcmsCIExyY WhitePoint,
 
 
 // Create profile from ICC file
-// TODO: For now, sAccess is ignored. Perhaps in a (far) future, I will consider
-// the creation of profiles.
-
-
-#ifdef __BORLANDC__
-#pragma warn -par
-#endif
-
-#ifdef _MSC_VER
-#pragma warning(disable : 4100)
-#endif
 
 cmsHPROFILE LCMSEXPORT cmsOpenProfileFromFile(const char *lpFileName, const char *sAccess)
 {
-#ifdef __MWERKS__
-#pragma unused(sAccess)
-#endif
-
        LPICCPROFILE NewIcc;
        FILE *ICCfile;
+	   cmsHPROFILE hEmpty;
+	   
+	   if (*sAccess == 'W' || *sAccess == 'w') {
+
+		   hEmpty = cmsCreateLabProfile(NULL);
+		   NewIcc = (LPICCPROFILE) (LPSTR) hEmpty;
+		   NewIcc -> IsWrite = TRUE;
+		   strncpy(NewIcc ->PhysicalFile, lpFileName, MAX_PATH-1);
+
+		   return hEmpty;
+	   }
 
 
        ICCfile = fopen(lpFileName, "rb");
        if (!ICCfile)
        {
               cmsSignalError(LCMS_ERRC_ABORTED, "File '%s' not found", lpFileName);
-              return NULL;  // Urecoverable
+              return NULL;  
        }
 
        NewIcc = ICCFileOpen(ICCfile);
@@ -519,6 +556,7 @@ cmsHPROFILE LCMSEXPORT cmsOpenProfileFromFile(const char *lpFileName, const char
 
        strncpy(NewIcc -> PhysicalFile, lpFileName, MAX_PATH-1);
        NewIcc -> IsTemporary = FALSE;
+	   NewIcc -> IsWrite     = FALSE;
 
        // in 32 bit, simply typecast
        return (cmsHPROFILE) (LPSTR) NewIcc;
@@ -597,9 +635,19 @@ BOOL LCMSEXPORT cmsCloseProfile(cmsHPROFILE hProfile)
 {
        LPICCPROFILE icco = (LPICCPROFILE) (LPSTR) hProfile;
        FILE *file;
+	   BOOL rc = TRUE;
 
        if (!icco) return FALSE;
 
+
+	   // Was open in write mode?	
+	   if (icco ->IsWrite) {
+
+		   icco ->IsWrite = FALSE;		// Assure no further writting
+		   rc = _cmsSaveProfile(hProfile, icco ->PhysicalFile);		   
+	   }
+
+
        file = icco -> stream;
 
        if (!file)
@@ -621,14 +669,13 @@ BOOL LCMSEXPORT cmsCloseProfile(cmsHPROFILE hProfile)
 
        free(icco);
 
-       return TRUE;
+       return rc;
 }
 
 
 
 // Is a table linear?
 
-
 int cmsIsLinear(WORD Table[], int nEntries)
 {
        register int i;
@@ -656,15 +703,75 @@ unsigned int uipow(unsigned int a, unsigned int b) {
 }
 
 
-// Convert between notations, as stated in ICC spec.
+// Convert between notations.
 
-#define TO16(x)      (WORD) ((x) << 8)
+#define TO16_TAB(x)      (WORD) (((x) << 8) | (x))
 
 
-// Case LUT 8
+// LUT8 can only come in Lab space. There is a fatal flaw in
+// converting from Lut8 to Lut16. Due to particular encoding 
+// of Lab, different actions should be taken from input and 
+// output Lab8 LUTS. For input, is as easy as applying a << 8,
+// since numbers comes in fixed point. However, for output LUT
+// things goes a bit more complex.... LUT 16 is supposed to
+// have a domain of 0..ff00, so we should remap the LUT in order
+// to get things working. Affected signatures are B2Axx tags, 
+// preview and gamut.
+
+// I do solve it by multiplying input matrix by:
+//	
+//	| 0xffff/0xff00   0    0  |
+//  |       0         1    0  |
+//  |       0         0    1  |
+//
+// The input values got then remapped to adequate domain
 
 static
-void ReadLUT8(LPICCPROFILE Icc, LPLUT NewLUT)
+void FixLUT8(LPLUT Lut, icTagSignature sig, int nTabSize)
+{	
+	MAT3 Fixup, Original, Result;
+	LPWORD PtrW;
+	int i;
+
+	switch (sig) {
+ 
+
+       case icSigBToA0Tag:
+       case icSigBToA1Tag:
+       case icSigBToA2Tag:
+       case icSigGamutTag:
+       case icSigPreview0Tag:
+       case icSigPreview1Tag:
+       case icSigPreview2Tag: 
+				
+				VEC3init(&Fixup.v[0], (double) 0xFFFF/0xFF00, 0, 0);
+				VEC3init(&Fixup.v[1], 0, 1, 0);
+				VEC3init(&Fixup.v[2], 0, 0, 1);
+
+				MAT3fromFix(&Original, &Lut->Matrix);				
+				MAT3per(&Result, &Original, &Fixup);
+				MAT3toFix(&Lut->Matrix, &Result);
+
+				Lut -> wFlags |= LUT_HASMATRIX;
+				break;
+
+	   // For input, clear low part since this has to be
+	   // Lab in fixed point
+
+	   default:        
+
+			    PtrW = Lut -> T;
+			    for (i = 0; i < nTabSize; i++) {
+
+							 *PtrW++ &= 0xFF00;
+				}
+	}
+	
+}
+
+// The infamous LUT 8
+static
+void ReadLUT8(LPICCPROFILE Icc, LPLUT NewLUT, icTagSignature sig)
 {
     icLut8 LUT8;
     LPBYTE Temp;
@@ -673,10 +780,8 @@ void ReadLUT8(LPICCPROFILE Icc, LPLUT NewLUT)
     unsigned int AllLinear;
     LPWORD PtrW;
 
-       fread(&LUT8, sizeof(icLut8) - SIZEOF_UINT8_ALIGNED, 1, Icc -> stream);
-
-       // fread(&LUT8, sizeof(my_icLut8), 1, Icc -> stream);
-
+       SafeRead(&LUT8, sizeof(icLut8) - SIZEOF_UINT8_ALIGNED, 1, Icc -> stream);
+       
        NewLUT -> wFlags        = LUT_HASTL1|LUT_HASTL2|LUT_HAS3DGRID;
        NewLUT -> cLutPoints    = LUT8.clutPoints;
        NewLUT -> InputChan     = LUT8.inputChan;
@@ -725,9 +830,9 @@ void ReadLUT8(LPICCPROFILE Icc, LPLUT NewLUT)
 
               PtrW = (LPWORD) malloc(sizeof(WORD) * 256);
               NewLUT -> L1[i] = PtrW;
-              fread(Temp, 1, 256, Icc -> stream);
+              SafeRead(Temp, 1, 256, Icc -> stream);
               for (j=0; j < 256; j++)
-                     PtrW[j] = TO16(Temp[j]);
+                     PtrW[j] = TO16_TAB(Temp[j]);
                      AllLinear += cmsIsLinear(NewLUT -> L1[i], NewLUT -> InputEntries);
               }
 
@@ -747,12 +852,12 @@ void ReadLUT8(LPICCPROFILE Icc, LPLUT NewLUT)
 
        PtrW = (LPWORD) malloc(sizeof(WORD) * nTabSize);
        Temp = (LPBYTE) malloc(nTabSize);
-       fread(Temp, 1, nTabSize, Icc -> stream);
+       SafeRead(Temp, 1, nTabSize, Icc -> stream);
 
        NewLUT -> T = PtrW;
        for (i = 0; i < nTabSize; i++) {
 
-                     *PtrW++ = TO16(Temp[i]);
+                     *PtrW++ = TO16_TAB(Temp[i]);
        }
        free(Temp);
 
@@ -765,9 +870,9 @@ void ReadLUT8(LPICCPROFILE Icc, LPLUT NewLUT)
 
               PtrW = (LPWORD) malloc(sizeof(WORD) * 256);
               NewLUT -> L2[i] = PtrW;
-              fread(Temp, 1, 256, Icc -> stream);
-              for (j=0; j < 256; j++)
-                     PtrW[j] = TO16(Temp[j]);
+              SafeRead(Temp, 1, 256, Icc -> stream);              
+			  for (j=0; j < 256; j++)
+                     PtrW[j] = TO16_TAB(Temp[j]);
                      AllLinear += cmsIsLinear(NewLUT -> L2[i], 256);
               }
 
@@ -786,13 +891,14 @@ void ReadLUT8(LPICCPROFILE Icc, LPLUT NewLUT)
        cmsCalcCLUT16Params(NewLUT -> cLutPoints,  NewLUT -> InputChan,
                                                   NewLUT -> OutputChan,
                                                   &NewLUT -> CLut16params);
+	   // Fixup
+	   FixLUT8(NewLUT, sig, nTabSize);
 
 }
 
 
 
 
-
 // Case LUT 16
 
 static
@@ -805,7 +911,7 @@ void ReadLUT16(LPICCPROFILE Icc, LPLUT NewLUT)
     LPWORD PtrW;
 
 
-       fread(&LUT16, sizeof(icLut16)- SIZEOF_UINT16_ALIGNED, 1, Icc -> stream);
+       SafeRead(&LUT16, sizeof(icLut16)- SIZEOF_UINT16_ALIGNED, 1, Icc -> stream);
 
        NewLUT -> wFlags        = LUT_HASTL1 | LUT_HASTL2 | LUT_HAS3DGRID;
        NewLUT -> cLutPoints    = LUT16.clutPoints;
@@ -856,7 +962,7 @@ void ReadLUT16(LPICCPROFILE Icc, LPLUT NewLUT)
 
               PtrW = (LPWORD) malloc(sizeof(WORD) * NewLUT -> InputEntries);
               NewLUT -> L1[i] = PtrW;
-              fread(PtrW, sizeof(WORD), NewLUT -> InputEntries, Icc -> stream);
+              SafeRead(PtrW, sizeof(WORD), NewLUT -> InputEntries, Icc -> stream);
               AdjustEndianessArray16(PtrW, NewLUT -> InputEntries);
               AllLinear += cmsIsLinear(NewLUT -> L1[i], NewLUT -> InputEntries);
               }
@@ -876,7 +982,7 @@ void ReadLUT16(LPICCPROFILE Icc, LPLUT NewLUT)
        PtrW = (LPWORD) malloc(sizeof(WORD) * nTabSize);
 
        NewLUT -> T = PtrW;
-       fread(PtrW, sizeof(WORD), nTabSize, Icc -> stream);
+       SafeRead(PtrW, sizeof(WORD), nTabSize, Icc -> stream);
        AdjustEndianessArray16(NewLUT -> T, nTabSize);
 
        // Copy output tables
@@ -886,7 +992,7 @@ void ReadLUT16(LPICCPROFILE Icc, LPLUT NewLUT)
 
               PtrW = (LPWORD) malloc(sizeof(WORD) * NewLUT -> OutputEntries);
               NewLUT -> L2[i] = PtrW;
-              fread(PtrW, sizeof(WORD), NewLUT -> OutputEntries, Icc -> stream);
+              SafeRead(PtrW, sizeof(WORD), NewLUT -> OutputEntries, Icc -> stream);
               AdjustEndianessArray16(PtrW, NewLUT -> OutputEntries);
               AllLinear += cmsIsLinear(NewLUT -> L2[i], NewLUT -> OutputEntries);
               }
@@ -908,7 +1014,7 @@ void ReadLUT16(LPICCPROFILE Icc, LPLUT NewLUT)
 
 
 
-LPLUT cmsReadICCLut(cmsHPROFILE hProfile, icTagSignature sig)
+LPLUT LCMSEXPORT cmsReadICCLut(cmsHPROFILE hProfile, icTagSignature sig)
 {
 
     LPICCPROFILE        Icc = (LPICCPROFILE) (LPSTR) hProfile;
@@ -944,7 +1050,7 @@ LPLUT cmsReadICCLut(cmsHPROFILE hProfile, icTagSignature sig)
        return NULL;
     }
 
-    fread(&Base, sizeof(icTagBase), 1, Icc -> stream);
+    SafeRead(&Base, sizeof(icTagBase), 1, Icc -> stream);
     AdjustEndianess32((LPBYTE) &Base.sig);
 
     NewLUT = cmsAllocLUT();
@@ -957,7 +1063,7 @@ LPLUT cmsReadICCLut(cmsHPROFILE hProfile, icTagSignature sig)
 
     switch (Base.sig) {
 
-    case icSigLut8Type: ReadLUT8(Icc, NewLUT); break;
+    case icSigLut8Type: ReadLUT8(Icc, NewLUT, sig); break;
     case icSigLut16Type: ReadLUT16(Icc, NewLUT); break;
 
     default:  cmsSignalError(LCMS_ERRC_ABORTED, "Bad tag signature %lx found.", Base.sig);
@@ -999,7 +1105,7 @@ int ReadICCAscii(cmsHPROFILE hProfile, icTagSignature sig, char *Name)
     size   = Icc -> TagSizes[n];
 
     fseek(Icc -> stream, offset, SEEK_SET);
-    fread(&Base, sizeof(icTagBase), 1, Icc -> stream);
+    SafeRead(&Base, sizeof(icTagBase), 1, Icc -> stream);
     AdjustEndianess32((LPBYTE) &Base.sig);
 
     switch (Base.sig)
@@ -1008,9 +1114,9 @@ int ReadICCAscii(cmsHPROFILE hProfile, icTagSignature sig, char *Name)
 
            icUInt32Number  AsciiCount;
 
-           fread(&AsciiCount, sizeof(icUInt32Number), 1, Icc -> stream);
+           SafeRead(&AsciiCount, sizeof(icUInt32Number), 1, Icc -> stream);
            AdjustEndianess32((LPBYTE) &AsciiCount);
-           fread(Name, 1, AsciiCount, Icc -> stream);
+           SafeRead(Name, 1, AsciiCount, Icc -> stream);
            size = AsciiCount;
            }
            break;
@@ -1020,7 +1126,7 @@ int ReadICCAscii(cmsHPROFILE hProfile, icTagSignature sig, char *Name)
     case icSigTextType:
 
            size -= sizeof(icTagBase);
-           fread(Name, 1, size, Icc -> stream);
+           SafeRead(Name, 1, size, Icc -> stream);
            break;
 
     default:
@@ -1063,7 +1169,7 @@ int ReadICCXYZ(cmsHPROFILE hProfile, icTagSignature sig, LPcmsCIEXYZ Value, BOOL
     offset = Icc -> TagOffsets[n];
 
     fseek(Icc -> stream, offset, SEEK_SET);
-    fread(&Base, 1, sizeof(icTagBase), Icc -> stream);
+    SafeRead(&Base, 1, sizeof(icTagBase), Icc -> stream);
     AdjustEndianess32((LPBYTE) &Base.sig);
 
     switch (Base.sig)
@@ -1072,7 +1178,7 @@ int ReadICCXYZ(cmsHPROFILE hProfile, icTagSignature sig, LPcmsCIEXYZ Value, BOOL
     case 0x7c3b10cL:	// Some apple broken embedded profiles does not have correct type	
     case icSigXYZType:
 
-           fread(&XYZ, sizeof(icXYZNumber), 1, Icc -> stream);
+           SafeRead(&XYZ, sizeof(icXYZNumber), 1, Icc -> stream);
            Value -> X = Convert15Fixed16(XYZ.X);
            Value -> Y = Convert15Fixed16(XYZ.Y);
            Value -> Z = Convert15Fixed16(XYZ.Z);
@@ -1095,9 +1201,9 @@ int ReadICCXYZ(cmsHPROFILE hProfile, icTagSignature sig, LPcmsCIEXYZ Value, BOOL
 static
 void NormalizeXYZ(LPcmsCIEXYZ Dest)
 {
-    while (Dest -> X > 1. &&
-           Dest -> Y > 1. &&
-           Dest -> Z > 1.) {
+    while (Dest -> X > 2. &&
+           Dest -> Y > 2. &&
+           Dest -> Z > 2.) {
 
                Dest -> X /= 10.;
                Dest -> Y /= 10.;
@@ -1190,7 +1296,7 @@ BOOL cdecl cmsReadICCMatrixRGB2XYZ(LPMAT3 r, cmsHPROFILE hProfile)
 }
 
 
-LPGAMMATABLE cdecl cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig)
+LPGAMMATABLE cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig)
 {
        LPICCPROFILE  Icc = (LPICCPROFILE) (LPSTR) hProfile;
        LPGAMMATABLE NewGamma;
@@ -1216,7 +1322,7 @@ LPGAMMATABLE cdecl cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig)
        offset = Icc -> TagOffsets[n];
 
        fseek(Icc -> stream, offset, SEEK_SET);
-       fread(&Base, 1, sizeof(icTagBase), Icc -> stream);
+       SafeRead(&Base, 1, sizeof(icTagBase), Icc -> stream);
        AdjustEndianess32((LPBYTE) &Base.sig);
 
 
@@ -1226,7 +1332,7 @@ LPGAMMATABLE cdecl cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig)
 	   case 0x9478ee00L:    // Monaco 2 profiler is BROKEN!
        case icSigCurveType:
 
-           fread(&Count, sizeof(icUInt32Number), 1, Icc -> stream);
+           SafeRead(&Count, sizeof(icUInt32Number), 1, Icc -> stream);
            AdjustEndianess32((LPBYTE) &Count);
 
 
@@ -1243,24 +1349,64 @@ LPGAMMATABLE cdecl cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig)
            case 1:  {
                      WORD SingleGammaFixed;
 
-                     fread(&SingleGammaFixed, sizeof(WORD), 1, Icc -> stream);
+                     SafeRead(&SingleGammaFixed, sizeof(WORD), 1, Icc -> stream);
                      AdjustEndianess16((LPBYTE) &SingleGammaFixed);
-                     return cmsBuildGamma(256, Convert8Fixed8(SingleGammaFixed));
+                     return cmsBuildGamma(4096, Convert8Fixed8(SingleGammaFixed));
                      }
 
-           default:  // Curve
-
+           default: { // Curve
+					
                      NewGamma = cmsAllocGamma(Count);
                      if (!NewGamma) return NULL;
 
-                     fread(NewGamma -> GammaTable, sizeof(WORD), Count, Icc -> stream);
+                     SafeRead(NewGamma -> GammaTable, sizeof(WORD), Count, Icc -> stream);
+					 
                      AdjustEndianessArray16(NewGamma -> GammaTable, Count);
 
                      return NewGamma;
+					}
               }
+		      break;
+
+	
+	   // Parametric curves
+	   case icSigParametricCurveType: {
+           
+		   int ParamsByType[] = { 1, 3, 4, 5, 7 };
+		   double Params[10];
+		   icS15Fixed16Number Num;
+		   icUInt32Number Reserved;
+		   icUInt16Number Reserved2;
+		   icUInt16Number   Type;
+		   int i;
+
+		   SafeRead(&Reserved, sizeof(icUInt32Number), 1, Icc -> stream);
+		   SafeRead(&Type, sizeof(icUInt16Number), 1, Icc -> stream);
+		   SafeRead(&Reserved2, sizeof(icUInt16Number), 1, Icc -> stream);
+		   
+		   AdjustEndianess16((LPBYTE) &Type);
+		   if (Type > 5) {
+
+				cmsSignalError(LCMS_ERRC_ABORTED, "Unknown parametric curve type '%d' found.", Type);
+				return NULL;
+		   }
+		
+		  ZeroMemory(Params, 10* sizeof(double));
+		  n = ParamsByType[Type];
+
+		  for (i=0; i < n; i++) {
+				SafeRead(&Num, sizeof(icS15Fixed16Number), 1, Icc -> stream);
+				Params[i] = Convert15Fixed16(Num);
+		  }
+
+
+		   NewGamma = cmsBuildParametricGamma(4096, Type, Params);
+		   return NewGamma;
+		  }
+	
 
        default:
-              cmsSignalError(LCMS_ERRC_ABORTED, "Bad tag signature %lx found.", Base.sig);
+              cmsSignalError(LCMS_ERRC_ABORTED, "Bad tag signature '%lx' found.", Base.sig);
               return NULL;
        }
 
@@ -1273,7 +1419,6 @@ LPGAMMATABLE cdecl cmsReadICCGamma(cmsHPROFILE hProfile, icTagSignature sig)
 
 icColorSpaceSignature LCMSEXPORT cmsGetPCS(cmsHPROFILE hProfile)
 {
-       // Simply typecast
        LPICCPROFILE  Icc = (LPICCPROFILE) (LPSTR) hProfile;
 
        return Icc -> PCS;
@@ -1309,7 +1454,6 @@ void LCMSEXPORT cmsSetColorSpace(cmsHPROFILE hProfile, icColorSpaceSignature sig
 
 BOOL LCMSEXPORT cmsIsTag(cmsHPROFILE hProfile, icTagSignature sig)
 {
-       // Simply typecast
        LPICCPROFILE  Icc = (LPICCPROFILE) (LPSTR) hProfile;
 
        return SearchTag(Icc, sig) >= 0;
@@ -1455,6 +1599,37 @@ const char*  LCMSEXPORT cmsTakeProductInfo(cmsHPROFILE hProfile)
        return Info;
 }
 
+// Extract the target data as a big string. Does not signal if tag is not present.
+
+BOOL LCMSEXPORT cmsTakeCharTargetData(cmsHPROFILE hProfile, char** Data, size_t* len)
+{
+	LPICCPROFILE  Icc = (LPICCPROFILE) (LPSTR) hProfile;
+	int n;
+
+	*Data = NULL;
+	*len  = 0;
+
+    n = SearchTag(Icc, icSigCharTargetTag);
+    if (n < 0) return FALSE;				
+
+    *len =  Icc -> TagSizes[n];
+	*Data = (char*) malloc(*len + 1);
+
+	if (!*Data) {
+
+		cmsSignalError(-1, "Out of memory allocating CharTarget space!");
+		return FALSE;
+	}
+
+	if (ReadICCAscii(hProfile, icSigCharTargetTag, *Data) < 0) 
+		return FALSE;
+
+	(*Data)[*len] = 0;	// Force a zero marker. Shouldn't be needed, but is 
+						// here for simplify things.
+
+	return TRUE;    
+}
+
 
 // Write profile ------------------------------------------------------------
 
@@ -1487,9 +1662,10 @@ BOOL DoWrite(FILE *OutStream, size_t size, void *Ptr)
 {
 	   if (size == 0) return TRUE;
 	   
+	   UsedSpace += size;
+
        if (OutStream == NULL) {
 
-              UsedSpace += size;
               return TRUE;
        }
 
@@ -1551,7 +1727,7 @@ BOOL SaveHeader(FILE *OutStream, LPICCPROFILE Icc)
        EncodeDateTime(&Header.date);
 
        Header.magic       = TransportValue32(icMagicNumber);
-       Header.platform    = TransportValue32(icSigMicrosoft);  // Sorry, I must put something here
+       Header.platform    = (icPlatformSignature)TransportValue32(icSigMicrosoft);  // Sorry, I must put something here
 
        Header.flags        = TransportValue32(Icc -> flags);
        Header.manufacturer = TransportValue32(lcmsSignature);
@@ -1592,7 +1768,6 @@ BOOL SetupBase(FILE *OutStream, icTagTypeSignature sig)
 }
 
 
-
 // Store an XYZ tag
 
 static
@@ -1607,6 +1782,7 @@ BOOL SaveXYZNumber(FILE *OutStream, LPcmsCIEXYZ Value)
     XYZ.Y = TransportValue32(DOUBLE_TO_FIXED(Value -> Y));
     XYZ.Z = TransportValue32(DOUBLE_TO_FIXED(Value -> Z));
 
+
     return DoWrite(OutStream, sizeof(icXYZNumber), &XYZ);
 }
 
@@ -1639,14 +1815,9 @@ BOOL SaveGamma(FILE *OutStream, LPGAMMATABLE Gamma)
 }
 
 
-// Align text - PhotoShop and other software does
-// expect this field aligned on qword boundary. This is not
-// required (and a note about this is not required can be
-// found in the icc spec) but we will try to make everyone happy.
 
 
-
-// Save an ASCII Tag 
+// Save an DESC Tag 
 
 static
 BOOL SaveDescription(FILE *OutStream, const char *Text)
@@ -1673,28 +1844,15 @@ BOOL SaveDescription(FILE *OutStream, const char *Text)
     return DoWrite(OutStream, 71, Filler);
 }
 
-
-// Save an ASCII Tag (short)
+// Save an ASCII Tag 
 
 static
 BOOL SaveText(FILE *OutStream, const char *Text)
 {
-    size_t len, AlignedSize, FillerSize;
-    char Filler[5];
-
-    len = strlen(Text) + 1;
+    size_t len = strlen(Text) + 1;
 
     if (!SetupBase(OutStream, icSigTextType)) return FALSE;
     if (!DoWrite(OutStream, len, (LPVOID) Text)) return FALSE;
-
-    AlignedSize = ALIGNLONG(len);
-    FillerSize  = AlignedSize - len;
-	if (FillerSize == 0) 
-			return TRUE;
-
-    ZeroMemory(Filler, 5);
-    if (!DoWrite(OutStream, FillerSize, Filler)) return FALSE;
-
     return TRUE;
 }
 
@@ -1716,15 +1874,17 @@ BOOL SaveOneChromaticity(FILE *OutStream, double x, double y)
 }
 
 
-// New tag added in Addendum II
+// New tag added in Addendum II of old spec.
 
 static
 BOOL SaveChromaticities(FILE *OutStream, LPcmsCIExyYTRIPLE chrm)
 {
        WORD nChans, Table;
+	   DWORD Reserved = 0;
 
        if (!SetupBase(OutStream, icSigChromaticityType)) return FALSE;
 
+	   if (!DoWrite(OutStream, sizeof(DWORD) , &Reserved)) return FALSE;
        nChans = TransportValue16(3);
        if (!DoWrite(OutStream, sizeof(WORD) , &nChans)) return FALSE;
        Table =  TransportValue16(0);
@@ -1838,7 +1998,7 @@ BOOL SaveTagDirectory(FILE *OutStream, LPICCPROFILE Icc)
 
        for (i=0; i < Icc -> TagCount; i++) {
 
-          Tag.sig    = TransportValue32(Icc -> TagNames[i]);
+          Tag.sig    = (icTagSignature)TransportValue32(Icc -> TagNames[i]);
           Tag.offset = TransportValue32(Icc -> TagOffsets[i]);
           Tag.size   = TransportValue32(Icc -> TagSizes[i]);
 
@@ -1858,9 +2018,25 @@ BOOL SaveTags(FILE *OutStream, LPICCPROFILE Icc)
     LPBYTE Data;
     icInt32Number i;
     size_t Begin;
+	size_t AlignedSpace, FillerSize;
+
 
     for (i=0; i < Icc -> TagCount; i++) {
 
+		
+        // Align to DWORD boundary, following new spec.
+		
+		AlignedSpace = ALIGNLONG(UsedSpace);
+        FillerSize  = AlignedSpace - UsedSpace;
+		if (FillerSize > 0)  {
+			
+			BYTE Filler[20];
+
+			ZeroMemory(Filler, 16);
+		    if (!DoWrite(OutStream, FillerSize, Filler)) return FALSE;
+		}
+		
+		
        Icc -> TagOffsets[i] = Begin = UsedSpace;
        Data = (LPBYTE) Icc -> TagPtrs[i];
        if (!Data)
@@ -1868,7 +2044,9 @@ BOOL SaveTags(FILE *OutStream, LPICCPROFILE Icc)
 
        switch (Icc -> TagNames[i]) {
 
-       case icSigProfileDescriptionTag:
+       case icSigProfileDescriptionTag: 
+	   case icSigDeviceMfgDescTag:
+       case icSigDeviceModelDescTag:
               if (!SaveDescription(OutStream, (const char *) Data)) return FALSE;
               break;
 
@@ -1876,7 +2054,7 @@ BOOL SaveTags(FILE *OutStream, LPICCPROFILE Icc)
        case icSigGreenColorantTag:
        case icSigBlueColorantTag:
        case icSigMediaWhitePointTag:
-       case icSigMediaBlackPointTag:
+       case icSigMediaBlackPointTag:		   
                if (!SaveXYZNumber(OutStream, (LPcmsCIEXYZ) Data)) return FALSE;
                break;
 
@@ -1888,9 +2066,8 @@ BOOL SaveTags(FILE *OutStream, LPICCPROFILE Icc)
                if (!SaveGamma(OutStream, (LPGAMMATABLE) Data)) return FALSE;
                break;
 
-       case icSigCopyrightTag:
-       case icSigDeviceMfgDescTag:
-       case icSigDeviceModelDescTag:
+	   case icSigCharTargetTag:
+       case icSigCopyrightTag:      
               if (!SaveText(OutStream, (const char *) Data)) return FALSE;
               break;
 
@@ -1898,7 +2075,7 @@ BOOL SaveTags(FILE *OutStream, LPICCPROFILE Icc)
               if (!SaveChromaticities(OutStream, (LPcmsCIExyYTRIPLE) Data)) return FALSE;
               break;
 
-       // Save LUT (new to 1.08)
+       // Save LUT 
 
        case icSigAToB0Tag:
        case icSigAToB1Tag:
@@ -1921,6 +2098,8 @@ BOOL SaveTags(FILE *OutStream, LPICCPROFILE Icc)
        Icc -> TagSizes[i] = (UsedSpace - Begin);
        }
 
+		
+
        return TRUE;
 }
 
@@ -1951,19 +2130,86 @@ BOOL LCMSEXPORT _cmsAddLUTTag(cmsHPROFILE hProfile, icTagSignature sig, void* lu
 }
 
 
-// EXPERIMENTAL: Only works on some profiles 
+BOOL LCMSEXPORT _cmsAddGammaTag(cmsHPROFILE hProfile, icTagSignature sig, LPGAMMATABLE TransferFunction)
+{
+	LPICCPROFILE Icc = (LPICCPROFILE) (LPSTR) hProfile;
 
+	InitTag(Icc, sig, SizeOfGammaTab(TransferFunction), TransferFunction);
+	return TRUE;
+}
+
+
+BOOL LCMSEXPORT _cmsAddChromaticityTag(cmsHPROFILE hProfile, icTagSignature sig, LPcmsCIExyYTRIPLE Chrm)
+{
+	LPICCPROFILE Icc = (LPICCPROFILE) (LPSTR) hProfile;
+
+	InitTag(Icc, sig, sizeof(cmsCIExyYTRIPLE), Chrm);
+	return TRUE;
+}
+
+
+// Add tags to profile structure
+
+BOOL LCMSEXPORT cmsAddTag(cmsHPROFILE hProfile, icTagSignature sig, void* Tag)
+{
+   switch (sig) {
+
+	   case icSigCharTargetTag:
+       case icSigCopyrightTag:             
+       case icSigProfileDescriptionTag: 
+	   case icSigDeviceMfgDescTag:
+       case icSigDeviceModelDescTag:
+              return _cmsAddTextTag(hProfile, sig, (const char*) Tag);
+
+       case icSigRedColorantTag:
+       case icSigGreenColorantTag:
+       case icSigBlueColorantTag:
+       case icSigMediaWhitePointTag:
+       case icSigMediaBlackPointTag:		   
+			  return _cmsAddXYZTag(hProfile, sig, (const LPcmsCIEXYZ) Tag);
+               
+
+       case icSigRedTRCTag:
+       case icSigGreenTRCTag:
+       case icSigBlueTRCTag:
+       case icSigGrayTRCTag:
+              return _cmsAddGammaTag(hProfile, sig, (LPGAMMATABLE) Tag);
+	                 
+       case icSigAToB0Tag:
+       case icSigAToB1Tag:
+       case icSigAToB2Tag:
+       case icSigBToA0Tag:
+       case icSigBToA1Tag:
+       case icSigBToA2Tag:
+       case icSigGamutTag:
+       case icSigPreview0Tag:
+       case icSigPreview1Tag:
+       case icSigPreview2Tag:
+              return _cmsAddLUTTag(hProfile, sig, Tag);
+              
+
+	   case icSigChromaticityTag:
+			  return _cmsAddChromaticityTag(hProfile, sig, (LPcmsCIExyYTRIPLE) Tag);              
+        
+	   default:
+			cmsSignalError(-1, "cmsAddTag: Tag '%x' is unsupported", sig);
+			return FALSE;
+   }
+
+}
+
+// Low-level save
 BOOL LCMSEXPORT _cmsSaveProfile(cmsHPROFILE hProfile, const char* FileName)
 {
        FILE *OutStream;
        LPICCPROFILE Icc = (LPICCPROFILE) (LPSTR) hProfile;
 
+	  
        // Pass #1 does compute offsets
 
        if (!SaveHeader(NULL, Icc)) return FALSE;
        if (!SaveTagDirectory(NULL, Icc)) return FALSE;
        if (!SaveTags(NULL, Icc)) return FALSE;
-
        OutStream = fopen(FileName, "wb");
        if (!OutStream) return FALSE;
 
