@@ -30,10 +30,39 @@
 
 #include "lcms.h"
 
+//  Pipeline of LUT (including new revision 4.0 of ICC spec)
+//
+//  [Mat] -> [L1] -> [Mat2] -> [Ofs2] -> [L3] -> [CLUT] -> [L4] -> [Mat3] -> [Ofs3] -> [L2]
+//
+//  Some of these stages would be missing. This implements the totality of 
+//  combinations of old and new LUT types as follows:
+//
+//  Lut8 & Lut16
+//  ============
+//	   [Mat] -> [L1] -> [CLUT] -> [L2]
+//
+//  Mat2, Ofs2, L3, L3, Mat3, Ofs3 are missing
+//
+//  LutAToB
+//  ========
+//
+//  [L1] -> [CLUT] -> [L4] -> [Mat3] -> [Ofs3] -> [L2]
+//
+//	Mat, Mat2, Ofs2, L3 are missing
+//
+//  LutBToA 
+//  =======
+// 
+//	[L1] -> [Mat2] -> [Ofs2] -> [L3] -> [CLUT] -> [L2]
+//
+//  Mat, L4, Mat3, Ofs3 are missing
+//
+
+
 
 // Lut Creation & Destruction
 
-LPLUT cmsAllocLUT(void)
+LPLUT LCMSEXPORT cmsAllocLUT(void)
 {
        LPLUT NewLUT;
 
@@ -44,7 +73,7 @@ LPLUT cmsAllocLUT(void)
        return NewLUT;
 }
 
-void cmsFreeLUT(LPLUT Lut)
+void LCMSEXPORT cmsFreeLUT(LPLUT Lut)
 {
        unsigned int i;
 
@@ -79,7 +108,7 @@ unsigned int UIpow(unsigned int a, unsigned int b)
 }
 
 
-LPLUT cmsAlloc3DGrid(LPLUT NewLUT, int clutPoints, int inputChan, int outputChan)
+LPLUT LCMSEXPORT cmsAlloc3DGrid(LPLUT NewLUT, int clutPoints, int inputChan, int outputChan)
 {
     size_t nTabSize;
 
@@ -106,7 +135,7 @@ LPLUT cmsAlloc3DGrid(LPLUT NewLUT, int clutPoints, int inputChan, int outputChan
 
 
 
-LPLUT cmsAllocLinearTable(LPLUT NewLUT, LPGAMMATABLE Tables[], int nTable)
+LPLUT LCMSEXPORT cmsAllocLinearTable(LPLUT NewLUT, LPGAMMATABLE Tables[], int nTable)
 {
        unsigned int i;
        LPWORD PtrW;
@@ -137,6 +166,35 @@ LPLUT cmsAllocLinearTable(LPLUT NewLUT, LPGAMMATABLE Tables[], int nTable)
                }
                break;
 
+
+	   // 3 & 4 according new spec
+
+	   case 3:
+			   NewLUT -> wFlags |= LUT_HASTL3;
+               cmsCalcL16Params(Tables[0] -> nEntries, &NewLUT -> L3params);
+               NewLUT -> L3Entries = Tables[0] -> nEntries;
+
+               for (i=0; i < NewLUT -> InputChan; i++) {
+
+                     PtrW = (LPWORD) malloc(sizeof(WORD) * NewLUT -> L3Entries);
+                     NewLUT -> L3[i] = PtrW;
+                     CopyMemory(PtrW, Tables[i]->GammaTable, sizeof(WORD) * NewLUT -> L3Entries);
+               }
+               break;
+
+	   case 4:
+			   NewLUT -> wFlags |= LUT_HASTL4;
+               cmsCalcL16Params(Tables[0] -> nEntries, &NewLUT -> L4params);
+               NewLUT -> L4Entries = Tables[0] -> nEntries;
+               for (i=0; i < NewLUT -> OutputChan; i++) {
+
+                     PtrW = (LPWORD) malloc(sizeof(WORD) * NewLUT -> L4Entries);
+                     NewLUT -> L4[i] = PtrW;
+                     CopyMemory(PtrW, Tables[i]->GammaTable, sizeof(WORD) * NewLUT -> L4Entries);
+               }
+               break;
+			   
+
        default:;
        }
 
@@ -144,10 +202,48 @@ LPLUT cmsAllocLinearTable(LPLUT NewLUT, LPGAMMATABLE Tables[], int nTable)
 }
 
 
+// Set the LUT matrix
+
+LPLUT LCMSEXPORT cmsSetMatrixLUT(LPLUT Lut, LPMAT3 M)
+{
+		MAT3toFix(&Lut ->Matrix, M);
+
+		if (!MAT3isIdentity(&Lut->Matrix, 0.0001))
+			Lut ->wFlags |= LUT_HASMATRIX;
+
+		return Lut;
+}
+
+
+// Eval gray LUT having only one input channel 
+static
+void Eval1Input(WORD StageABC[], WORD StageLMN[], WORD LutTable[], LPL16PARAMS p16)
+{
+	   Fixed32 fk;
+       Fixed32 k0, k1, rk, K0, K1;
+	   int OutChan;
+
+	   fk = ToFixedDomain((Fixed32) StageABC[0] * p16 -> Domain);
+       k0 = FIXED_TO_INT(fk);
+       rk = (WORD) FIXED_REST_TO_INT(fk);
+
+	   k1 = k0 + (StageABC[0] != 0xFFFFU ? 1 : 0);
+
+	   K0 = p16 -> opta1 * k0;
+       K1 = p16 -> opta1 * k1;
+
+	   for (OutChan=0; OutChan < p16->nOutputs; OutChan++)
+	   {
+
+		   StageLMN[OutChan] = (WORD) FixedLERP(rk, LutTable[K0+OutChan],
+											        LutTable[K1+OutChan]);
+	   }
+}
+
+
+
 // For more that 3 inputs (i.e., CMYK)
 // evaluate two 3-dimensional interpolations and then linearly interpolate between them.
-
-
 static
 void Eval4Inputs(WORD StageABC[], WORD StageLMN[], WORD LutTable[], LPL16PARAMS p16)
 {       
@@ -156,7 +252,7 @@ void Eval4Inputs(WORD StageABC[], WORD StageLMN[], WORD LutTable[], LPL16PARAMS
        int K0, K1;
        LPWORD T;
        int i;
-       WORD Tmp1[3], Tmp2[3];
+       WORD Tmp1[MAXCHANNELS], Tmp2[MAXCHANNELS];
 
        
        fk = ToFixedDomain((Fixed32) StageABC[0] * p16 -> Domain);
@@ -200,7 +296,7 @@ void Eval5Inputs(WORD StageABC[], WORD StageLMN[], WORD LutTable[], LPL16PARAMS
        int K0, K1;
        LPWORD T;
        int i;
-       WORD Tmp1[3], Tmp2[3];
+       WORD Tmp1[MAXCHANNELS], Tmp2[MAXCHANNELS];
 
        
        fk = ToFixedDomain((Fixed32) StageABC[0] * p16 -> Domain);
@@ -238,7 +334,7 @@ void Eval6Inputs(WORD StageABC[], WORD StageLMN[], WORD LutTable[], LPL16PARAMS
        int K0, K1;
        LPWORD T;
        int i;
-       WORD Tmp1[3], Tmp2[3];
+       WORD Tmp1[MAXCHANNELS], Tmp2[MAXCHANNELS];
 
        
        fk = ToFixedDomain((Fixed32) StageABC[0] * p16 -> Domain);
@@ -267,7 +363,7 @@ void Eval6Inputs(WORD StageABC[], WORD StageLMN[], WORD LutTable[], LPL16PARAMS
 }
 
 
-void cmsEvalLUT(LPLUT Lut, WORD In[], WORD Out[])
+void LCMSEXPORT cmsEvalLUT(LPLUT Lut, WORD In[], WORD Out[])
 {
        register unsigned int i;
        WORD StageABC[MAXCHANNELS], StageLMN[MAXCHANNELS];
@@ -309,6 +405,9 @@ void cmsEvalLUT(LPLUT Lut, WORD In[], WORD Out[])
        }
 
 
+	   // TODO: Mat2, Ofs2, L3 processing
+
+
        if (Lut -> wFlags & LUT_HAS3DGRID)
        {
               // If it is 4 channels input (like CMYK, for example),
@@ -317,6 +416,12 @@ void cmsEvalLUT(LPLUT Lut, WORD In[], WORD Out[])
 
 		   switch (Lut -> InputChan) {
 
+
+		   case 1: // Gray LUT
+
+			   Eval1Input(StageABC, StageLMN, Lut->T, &Lut->CLut16params);
+			   break;
+
            case 3:
               
 #ifdef USE_TETRAHEDRAL
@@ -354,6 +459,8 @@ void cmsEvalLUT(LPLUT Lut, WORD In[], WORD Out[])
        }
 
 
+	   // TODO: Mat3, Ofs3, L3 processing
+
        // Last linearitzation
 
        if (Lut -> wFlags & LUT_HASTL2)
