@@ -42,6 +42,7 @@ void cdecl VEC3divK(LPVEC3 r, LPVEC3 v, double d);
 void cdecl VEC3perK(LPVEC3 r, LPVEC3 v, double d);
 void cdecl VEC3perComp(LPVEC3 r, LPVEC3 a, LPVEC3 b);
 void cdecl VEC3minus(LPVEC3 r, LPVEC3 a, LPVEC3 b);
+void cdecl VEC3scaleAndCut(LPWVEC3 r, LPVEC3 v, double d);
 
 void cdecl MAT3identity(LPMAT3 a);
 void cdecl MAT3per(LPMAT3 r, LPMAT3 a, LPMAT3 b);
@@ -50,7 +51,7 @@ void cdecl MAT3eval(LPVEC3 r, LPMAT3 a, LPVEC3 v);
 void cdecl MAT3toFix(LPWMAT3 r, LPMAT3 v);
 void cdecl MAT3evalW(LPWVEC3 r, LPWMAT3 a, LPWVEC3 v);
 void cdecl MAT3perK(LPMAT3 r, LPMAT3 v, double d);
-
+void cdecl MAT3scaleAndCut(LPWMAT3 r, LPMAT3 v, double d);
 
 // --------------------- Implementation ----------------------------
 
@@ -206,7 +207,7 @@ int FromFixedDomain(Fixed32 a)
 }
 
 
-// Linear IntREPolation
+// Linear intERPolation
 // a * (h - l) >> 16 + l
 
 Fixed32 FixedLERP(Fixed32 a, Fixed32 l, Fixed32 h)
@@ -351,6 +352,15 @@ void VEC3toFix(LPWVEC3 r, LPVEC3 v)
        r -> n[VZ] = DOUBLE_TO_FIXED(v -> n[VZ]);
 }
 
+// Convert from fixed point
+
+void VEC3fromFix(LPVEC3 r, LPWVEC3 v)
+{
+       r -> n[VX] = FIXED_TO_DOUBLE(v -> n[VX]);
+       r -> n[VY] = FIXED_TO_DOUBLE(v -> n[VY]);
+       r -> n[VZ] = FIXED_TO_DOUBLE(v -> n[VZ]);
+}
+
 
 // Swap two double vectors
 
@@ -551,16 +561,6 @@ void MAT3eval(LPVEC3 r, LPMAT3 a, LPVEC3 v)
 
 #ifdef USE_ASSEMBLER
 
-// MMX optimized matrix per vector, uses saturation mode
-#ifdef USE_ASSEMBLER_MMX
-void MAT3evalW(LPWVEC3 r_, LPWMAT3 a_, LPWVEC3 v_)
-{
-       ASM {
-
-       }
-}
-
-#else
 
 // ecx:ebx is result in 64 bits format
 // edi points to matrix, esi points to input vector
@@ -689,7 +689,7 @@ void MAT3evalW(LPWVEC3 r_, LPWMAT3 a_, LPWVEC3 v_)
        mov       dword ptr [edi+8], ecx      // r -> n[VZ]
        }
 }
-#endif
+
 
 #else
 void MAT3evalW(LPWVEC3 r, LPWMAT3 a, LPWVEC3 v)
@@ -725,6 +725,12 @@ void MAT3toFix(LPWMAT3 r, LPMAT3 v)
        VEC3toFix(&r -> v[2], &v -> v[2]);
 }
 
+void MAT3fromFix(LPMAT3 r, LPWMAT3 v)
+{
+       VEC3fromFix(&r -> v[0], &v -> v[0]);
+       VEC3fromFix(&r -> v[1], &v -> v[1]);
+       VEC3fromFix(&r -> v[2], &v -> v[2]);
+}
 
 
 
