@@ -42,7 +42,7 @@
 
 // These macros handles reverse flavor
 
-#define REVERSE_FLAVOR_8(x)	 ((BYTE) (0xff-(x)))
+#define REVERSE_FLAVOR_8(x)     ((BYTE) (0xff-(x)))
 #define REVERSE_FLAVOR_16(x)    ((WORD)(0xffff-(x)))
 
 // Supress waning about info never being used
@@ -65,8 +65,8 @@ LPBYTE Unroll6Bytes(register _LPcmsTRANSFORM info, register WORD wIn[], register
        wIn[1] = RGB_8_TO_16(*accum); accum++; 
        wIn[2] = RGB_8_TO_16(*accum); accum++; 
        wIn[3] = RGB_8_TO_16(*accum); accum++; 
-	   wIn[4] = RGB_8_TO_16(*accum); accum++; 
-	   wIn[5] = RGB_8_TO_16(*accum); accum++; 
+       wIn[4] = RGB_8_TO_16(*accum); accum++; 
+       wIn[5] = RGB_8_TO_16(*accum); accum++; 
 
 
        return accum;
@@ -90,7 +90,7 @@ static
 LPBYTE Unroll4BytesSwapFirst(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
 
-	   wIn[3] = RGB_8_TO_16(*accum); accum++; // K
+       wIn[3] = RGB_8_TO_16(*accum); accum++; // K
        wIn[0] = RGB_8_TO_16(*accum); accum++; // C
        wIn[1] = RGB_8_TO_16(*accum); accum++; // M
        wIn[2] = RGB_8_TO_16(*accum); accum++; // Y
@@ -133,8 +133,8 @@ LPBYTE Unroll6Words(register _LPcmsTRANSFORM info, register WORD wIn[], register
        wIn[1] = *(LPWORD) accum; accum+= 2; 
        wIn[2] = *(LPWORD) accum; accum+= 2; 
        wIn[3] = *(LPWORD) accum; accum+= 2; 
-	   wIn[4] = *(LPWORD) accum; accum+= 2; 
-	   wIn[5] = *(LPWORD) accum; accum+= 2; 
+       wIn[4] = *(LPWORD) accum; accum+= 2; 
+       wIn[5] = *(LPWORD) accum; accum+= 2; 
 
        return accum;
 }
@@ -155,7 +155,7 @@ static
 LPBYTE Unroll4WordsSwapFirst(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
        wIn[3] = *(LPWORD) accum; accum+= 2; // K
-	   wIn[0] = *(LPWORD) accum; accum+= 2; // C
+       wIn[0] = *(LPWORD) accum; accum+= 2; // C
        wIn[1] = *(LPWORD) accum; accum+= 2; // M
        wIn[2] = *(LPWORD) accum; accum+= 2; // Y
        
@@ -285,6 +285,15 @@ LPBYTE Unroll1Byte(register _LPcmsTRANSFORM info, register WORD wIn[], register
        return accum;
 }
 
+
+static
+LPBYTE Unroll1ByteSkip2(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
+{
+       wIn[0] = wIn[1] = wIn[2] = RGB_8_TO_16(*accum); accum++;     // L
+       accum += 2;
+       return accum;
+}
+
 static
 LPBYTE Unroll1ByteReversed(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
@@ -315,6 +324,16 @@ LPBYTE Unroll1WordBigEndian(register _LPcmsTRANSFORM info, register WORD wIn[],
        return accum;
 }
 
+static
+LPBYTE Unroll1WordSkip3(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
+{
+       wIn[0] = wIn[1] = wIn[2] = *accum; 
+        
+       accum += 8;
+       return accum;
+}
+
+
 // Monochrome + alpha. Alpha is lost
 
 static
@@ -328,7 +347,7 @@ LPBYTE Unroll2Byte(register _LPcmsTRANSFORM info, register WORD wIn[], register
 static
 LPBYTE Unroll2ByteSwapFirst(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
-	   wIn[3] = RGB_8_TO_16(*accum); accum++;                       // alpha
+       wIn[3] = RGB_8_TO_16(*accum); accum++;                       // alpha
        wIn[0] = wIn[1] = wIn[2] = RGB_8_TO_16(*accum); accum++;     // L       
        return accum;
 }
@@ -347,7 +366,7 @@ LPBYTE Unroll2Word(register _LPcmsTRANSFORM info, register WORD wIn[], register
 static
 LPBYTE Unroll2WordSwapFirst(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
-	   wIn[3] = *(LPWORD) accum; accum += 2;                    // alpha
+       wIn[3] = *(LPWORD) accum; accum += 2;                    // alpha
        wIn[0] = wIn[1] = wIn[2] = *(LPWORD) accum; accum+= 2;   // L
        
        return accum;
@@ -363,6 +382,8 @@ LPBYTE Unroll2WordBigEndian(register _LPcmsTRANSFORM info, register WORD wIn[],
 }
 
 
+
+
 static
 LPBYTE UnrollPlanarBytes(register _LPcmsTRANSFORM info, register WORD wIn[], register LPBYTE accum)
 {
@@ -505,6 +526,39 @@ LPBYTE PackNWordsSwapBigEndian(register _LPcmsTRANSFORM info, register WORD wOut
 }
 
 
+static
+LPBYTE PackPlanarBytes(register _LPcmsTRANSFORM info, register WORD wOut[], register LPBYTE output)
+{
+       int nChan = T_CHANNELS(info -> OutputFormat);
+       register int i;
+       LPBYTE Init = output;
+
+       for (i=0; i < nChan; i++) {
+
+              *(LPBYTE) output = RGB_16_TO_8(wOut[i]);
+              output += info -> StrideOut;
+       }
+
+       return (Init + 1);
+}
+
+
+static
+LPBYTE PackPlanarWords(register _LPcmsTRANSFORM info, register WORD wOut[], register LPBYTE output)
+{
+       int nChan = T_CHANNELS(info -> OutputFormat);
+       register int i;
+       LPBYTE Init = output;
+
+       for (i=0; i < nChan; i++) {
+
+              *(LPWORD) output = wOut[i];
+              output += (info -> StrideOut * sizeof(WORD));
+       }
+
+       return (Init + 2);
+}
+
 
 // CMYKcm (unrolled for speed)
 
@@ -631,7 +685,7 @@ LPBYTE Pack4Bytes(register _LPcmsTRANSFORM info, register WORD wOut[], register
 static
 LPBYTE Pack4BytesSwapFirst(register _LPcmsTRANSFORM info, register WORD wOut[], register LPBYTE output)
 {
-		 *output++ = RGB_16_TO_8(wOut[3]);
+         *output++ = RGB_16_TO_8(wOut[3]);
          *output++ = RGB_16_TO_8(wOut[0]);
          *output++ = RGB_16_TO_8(wOut[1]);
          *output++ = RGB_16_TO_8(wOut[2]);
@@ -821,7 +875,7 @@ LPBYTE Pack3BytesAndSkip1(register _LPcmsTRANSFORM Info, register WORD wOut[], r
 static
 LPBYTE Pack3BytesAndSkip1SwapFirst(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
 {
-		output++;
+        output++;
        *output++ = RGB_16_TO_8(wOut[0]);
        *output++ = RGB_16_TO_8(wOut[1]);
        *output++ = RGB_16_TO_8(wOut[2]);
@@ -832,7 +886,7 @@ LPBYTE Pack3BytesAndSkip1SwapFirst(register _LPcmsTRANSFORM Info, register WORD
 static
 LPBYTE Pack3BytesAndSkip1Swap(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
 {
-		output++;
+        output++;
        *output++ = RGB_16_TO_8(wOut[2]);
        *output++ = RGB_16_TO_8(wOut[1]);
        *output++ = RGB_16_TO_8(wOut[0]);
@@ -843,11 +897,11 @@ LPBYTE Pack3BytesAndSkip1Swap(register _LPcmsTRANSFORM Info, register WORD wOut[
 
 static
 LPBYTE Pack3BytesAndSkip1SwapSwapFirst(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
-{		
+{       
        *output++ = RGB_16_TO_8(wOut[2]);
        *output++ = RGB_16_TO_8(wOut[1]);
        *output++ = RGB_16_TO_8(wOut[0]);
-	   output++;
+       output++;
     
        return output;
 }
@@ -870,7 +924,7 @@ LPBYTE Pack3WordsAndSkip1(register _LPcmsTRANSFORM Info, register WORD wOut[], r
 static
 LPBYTE Pack3WordsAndSkip1Swap(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
 {
-	   output+= 2;
+       output+= 2;
        *(LPWORD) output = wOut[2];
        output+= 2;
        *(LPWORD) output = wOut[1];
@@ -885,14 +939,14 @@ LPBYTE Pack3WordsAndSkip1Swap(register _LPcmsTRANSFORM Info, register WORD wOut[
 
 static
 LPBYTE Pack3WordsAndSkip1SwapSwapFirst(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
-{	   
+{      
        *(LPWORD) output = wOut[2];
        output+= 2;
        *(LPWORD) output = wOut[1];
        output+= 2;
        *(LPWORD) output = wOut[0];
        output+= 2;
-	   output+= 2;
+       output+= 2;
        
 
        return output;
@@ -917,7 +971,7 @@ LPBYTE Pack3WordsAndSkip1BigEndian(register _LPcmsTRANSFORM Info, register WORD
 static
 LPBYTE Pack3WordsAndSkip1SwapBigEndian(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
 {
-		output+= 2;
+        output+= 2;
        *(LPWORD) output = CHANGE_ENDIAN(wOut[2]);
        output+= 2;
        *(LPWORD) output = CHANGE_ENDIAN(wOut[1]);
@@ -951,7 +1005,7 @@ LPBYTE Pack1ByteAndSkip1(register _LPcmsTRANSFORM Info, register WORD wOut[], re
 static
 LPBYTE Pack1ByteAndSkip1SwapFirst(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
 {
-	   output++;
+       output++;
        *output++ = RGB_16_TO_8(wOut[0]);
        
        return output;
@@ -988,7 +1042,7 @@ LPBYTE Pack1WordAndSkip1(register _LPcmsTRANSFORM Info, register WORD wOut[], re
 static
 LPBYTE Pack1WordAndSkip1SwapFirst(register _LPcmsTRANSFORM Info, register WORD wOut[], register LPBYTE output)
 {
-	   output += 2;	
+       output += 2; 
        *(LPWORD) output = wOut[0];
        output+= 2;
 
@@ -1040,42 +1094,46 @@ _cmsFIXFN _cmsIdentifyInputFormat(DWORD dwInput)
               case 1: if (T_FLAVOR(dwInput))
                                 FromInput = Unroll1ByteReversed;
                             else
-                            	  FromInput = Unroll1Byte;
+                                  FromInput = Unroll1Byte;
                       break;
 
               case 2: if (T_SWAPFIRST(dwInput))
-						FromInput = Unroll2ByteSwapFirst;
-					  else
-						FromInput = Unroll2Byte;
+                        FromInput = Unroll2ByteSwapFirst;
+                      else
+                        FromInput = Unroll2Byte;
                       break;
 
               case 3: if (T_DOSWAP(dwInput))
                             FromInput = Unroll3BytesSwap;
-                      else
-                            FromInput = Unroll3Bytes;
+                      else {
+                            if (T_EXTRA(dwInput) == 2)
+                                FromInput = Unroll1ByteSkip2;
+                            else
+                                FromInput = Unroll3Bytes;
+                      }
                       break;
               case 4:
-					  if (T_DOSWAP(dwInput)) {
-						    if (T_SWAPFIRST(dwInput))
+                      if (T_DOSWAP(dwInput)) {
+                            if (T_SWAPFIRST(dwInput))
 
-								FromInput = Unroll4BytesSwapSwapFirst;
-							else 
-								FromInput = Unroll4BytesSwap;
-					  }
+                                FromInput = Unroll4BytesSwapSwapFirst;
+                            else 
+                                FromInput = Unroll4BytesSwap;
+                      }
                       else {
-						    if (T_SWAPFIRST(dwInput))
-								FromInput = Unroll4BytesSwapFirst;
-							else
-								FromInput = Unroll4Bytes;
-					  }
+                            if (T_SWAPFIRST(dwInput))
+                                FromInput = Unroll4BytesSwapFirst;
+                            else
+                                FromInput = Unroll4Bytes;
+                      }
                       break;
 
-			  case 6:
-				   if (!T_DOSWAP(dwInput) && !T_SWAPFIRST(dwInput))
-					   FromInput = Unroll6Bytes;
-				   break;
+              case 6:
+                   if (!T_DOSWAP(dwInput) && !T_SWAPFIRST(dwInput))
+                       FromInput = Unroll6Bytes;
+                   break;
 
-					
+                    
               default:;
               }
               break;
@@ -1091,17 +1149,17 @@ _cmsFIXFN _cmsIdentifyInputFormat(DWORD dwInput)
                           if (T_FLAVOR(dwInput))
                                 FromInput = Unroll1WordReversed;
                             else
-                            	  FromInput = Unroll1Word;
+                                  FromInput = Unroll1Word;
                       break;
 
               case 2: if (T_ENDIAN16(dwInput))
                             FromInput = Unroll2WordBigEndian;
-						else {
-						  if (T_SWAPFIRST(dwInput))
-							  FromInput = Unroll2WordSwapFirst;
-						  else 
-                              FromInput = Unroll2Word;				
-						}
+                        else {
+                          if (T_SWAPFIRST(dwInput))
+                              FromInput = Unroll2WordSwapFirst;
+                          else 
+                              FromInput = Unroll2Word;              
+                        }
                         break;
 
               case 3: if (T_DOSWAP(dwInput)) {
@@ -1122,30 +1180,35 @@ _cmsFIXFN _cmsIdentifyInputFormat(DWORD dwInput)
                             if (T_ENDIAN16(dwInput))
                                    FromInput = Unroll4WordsSwapBigEndian;
                             else {
-								   
-									if (T_SWAPFIRST(dwInput))
-										FromInput = Unroll4WordsSwapSwapFirst;
-									else
-										FromInput = Unroll4WordsSwap;
-							}
-							
+                                   
+                                    if (T_SWAPFIRST(dwInput))
+                                        FromInput = Unroll4WordsSwapSwapFirst;
+                                    else
+                                        FromInput = Unroll4WordsSwap;
+                            }
+                            
                       }
                       else {
+
+                            if (T_EXTRA(dwInput) == 3)
+                                    FromInput = Unroll1WordSkip3;
+                            else
+
                             if (T_ENDIAN16(dwInput))
                                    FromInput = Unroll4WordsBigEndian;
                             else {
-								  if (T_SWAPFIRST(dwInput))
-									FromInput = Unroll4WordsSwapFirst;
-								   else	
-									FromInput = Unroll4Words;
-							}
+                                  if (T_SWAPFIRST(dwInput))
+                                    FromInput = Unroll4WordsSwapFirst;
+                                   else 
+                                    FromInput = Unroll4Words;
+                            }
                       }
                       break;
 
-			  case 6:
-					if (!T_DOSWAP(dwInput) && !T_SWAPFIRST(dwInput))
-					   FromInput = Unroll6Words;
-				    break;
+              case 6:
+                    if (!T_DOSWAP(dwInput) && !T_SWAPFIRST(dwInput))
+                       FromInput = Unroll6Words;
+                    break;
 
               }
               break;
@@ -1164,7 +1227,23 @@ _cmsFIXFN _cmsIdentifyInputFormat(DWORD dwInput)
 
 _cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
 {
-              _cmsFIXFN ToOutput = NULL;
+       _cmsFIXFN ToOutput = NULL;
+
+       if (T_PLANAR(dwOutput)) {
+
+       switch (T_BYTES(dwOutput)) {
+
+              case 1: ToOutput = PackPlanarBytes;
+                      break;
+
+              case 2:if (!T_ENDIAN16(dwOutput))
+                            ToOutput = PackPlanarWords;
+                      break;
+
+              default:;
+       }
+       }
+       else {
 
               switch (T_BYTES(dwOutput)) {
 
@@ -1174,11 +1253,11 @@ _cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
                      case 1:
                             ToOutput = Pack1Byte;
                             if (T_EXTRA(dwOutput) == 1) {
-								if (T_SWAPFIRST(dwOutput))
-								   ToOutput = Pack1ByteAndSkip1SwapFirst;
-								else
+                                if (T_SWAPFIRST(dwOutput))
+                                   ToOutput = Pack1ByteAndSkip1SwapFirst;
+                                else
                                    ToOutput = Pack1ByteAndSkip1;
-							}
+                            }
                             break;
 
                      case 3:
@@ -1192,17 +1271,17 @@ _cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
 
                          case 1: if (T_DOSWAP(dwOutput)) {
 
-									if (T_SWAPFIRST(dwOutput)) 	
-										ToOutput = Pack3BytesAndSkip1SwapSwapFirst;
-									else
-										ToOutput = Pack3BytesAndSkip1Swap;
-								 }
+                                    if (T_SWAPFIRST(dwOutput))  
+                                        ToOutput = Pack3BytesAndSkip1SwapSwapFirst;
+                                    else
+                                        ToOutput = Pack3BytesAndSkip1Swap;
+                                 }
                              else {
-								   if (T_SWAPFIRST(dwOutput))
-									ToOutput = Pack3BytesAndSkip1SwapFirst;
-								   else 
-									ToOutput = Pack3BytesAndSkip1;
-							 }
+                                   if (T_SWAPFIRST(dwOutput))
+                                    ToOutput = Pack3BytesAndSkip1SwapFirst;
+                                   else 
+                                    ToOutput = Pack3BytesAndSkip1;
+                             }
                              break;
 
                          default:;
@@ -1211,18 +1290,18 @@ _cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
 
                      case 4: if (T_EXTRA(dwOutput) == 0)
                              {
-							 if (T_DOSWAP(dwOutput)) {
-								 if (T_SWAPFIRST(dwOutput))
-										ToOutput = Pack4BytesSwapSwapFirst;
-								 else
-										ToOutput = Pack4BytesSwap;
-							 }
+                             if (T_DOSWAP(dwOutput)) {
+                                 if (T_SWAPFIRST(dwOutput))
+                                        ToOutput = Pack4BytesSwapSwapFirst;
+                                 else
+                                        ToOutput = Pack4BytesSwap;
+                             }
                              else {
-								 if (T_SWAPFIRST(dwOutput))
-										ToOutput = Pack4BytesSwapFirst;
-								 else
-										ToOutput = Pack4Bytes;
-							 }
+                                 if (T_SWAPFIRST(dwOutput))
+                                        ToOutput = Pack4BytesSwapFirst;
+                                 else
+                                        ToOutput = Pack4Bytes;
+                             }
                              }
                              break;
 
@@ -1277,11 +1356,11 @@ _cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
 
                                    ToOutput = Pack1WordAndSkip1BigEndian;
                                else {
-								   if (T_SWAPFIRST(dwOutput))
-									  ToOutput = Pack1WordAndSkip1SwapFirst;
-								   else 
+                                   if (T_SWAPFIRST(dwOutput))
+                                      ToOutput = Pack1WordAndSkip1SwapFirst;
+                                   else 
                                       ToOutput = Pack1WordAndSkip1;
-							   }
+                               }
                             }
                             break;
 
@@ -1313,11 +1392,11 @@ _cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
 
                                           ToOutput = Pack3WordsAndSkip1SwapBigEndian;
                                    else {
-									   if (T_SWAPFIRST(dwOutput)) 
-										  ToOutput = Pack3WordsAndSkip1SwapSwapFirst;
-									   else 
+                                       if (T_SWAPFIRST(dwOutput)) 
+                                          ToOutput = Pack3WordsAndSkip1SwapSwapFirst;
+                                       else 
                                           ToOutput = Pack3WordsAndSkip1Swap;
-								   }
+                                   }
                              }
                              else  {
                                    if (T_ENDIAN16(dwOutput))
@@ -1401,6 +1480,7 @@ _cmsFIXFN _cmsIdentifyOutputFormat(DWORD dwOutput)
 
               default:;
               }
+              }
 
               if (!ToOutput)
                      cmsSignalError(LCMS_ERRC_ABORTED, "Unknown output format");
