@@ -196,7 +196,7 @@ void NormalXFORM(_LPcmsTRANSFORM p,
 
        if (p -> Stage1) {
 
-              p -> Stage1(wStageABC, wPCS, &p->a1, &p->b1);
+              p -> Stage1(wStageABC, wPCS, &p->m1, &p->of1);
 
               if (wPCS[0] == 0xFFFF &&
                   wPCS[1] == 0xFFFF &&
@@ -220,9 +220,14 @@ void NormalXFORM(_LPcmsTRANSFORM p,
        // Gamut check, enabled across CLUT
 
        cmsEvalLUT(p -> Gamut, wPCS, wGamut);
+       
+#ifdef DEBUG
+         printf("Gamut %d\n", wGamut[0]);
+#endif
 
-       if (wGamut[0] > 0x9A) {              // 1 for roundoff
-
+       if (wGamut[0] >= 150) {              // 1a for roundoff
+        
+              
               wOut[0] = AlarmR;          // Gamut alarm
               wOut[1] = AlarmG;
               wOut[2] = AlarmB;
@@ -242,7 +247,7 @@ void NormalXFORM(_LPcmsTRANSFORM p,
        }
 
        if (p -> Stage2) {
-              p -> Stage2(wPCS, wStageLMN, &p->a2, &p->b2);
+              p -> Stage2(wPCS, wStageLMN, &p->m2, &p->of2);
               if (wPCS[0] == 0xFFFF &&
                   wPCS[1] == 0xFFFF &&
                   wPCS[2] == 0xFFFF) {
@@ -408,8 +413,6 @@ BOOL cmsBuildOutputMatrixShaper(_LPcmsTRANSFORM p, LPDWORD dwFlags)
 
        if (cmsGetColorSpace(p -> OutputProfile) == icSigGrayData)
        {
-             // cmsSignalError(LCMS_ERRC_WARNING, "Sorry, output grayscale profiles not supported");
-
               *dwFlags |= cmsFLAGS_NOTPRECALC;
               p -> OutMatShaper = cmsBuildGrayMatrixShaper(p-> OutputProfile, MATSHAPER_OUTPUT);
               return p -> OutMatShaper != NULL;
@@ -474,6 +477,7 @@ BOOL cmsBuildSmeltMatShaper(_LPcmsTRANSFORM p)
        LPGAMMATABLE Shapes[3];
        int i;
 
+		// VEC3 wp, wr, wf;
 
        if (!cmsReadICCMatrixRGB2XYZ(&From, p -> InputProfile))
                      return FALSE;
@@ -482,15 +486,29 @@ BOOL cmsBuildSmeltMatShaper(_LPcmsTRANSFORM p)
        if (!cmsReadICCMatrixRGB2XYZ(&To, p -> OutputProfile))
                      return FALSE;
 
+	   
 
+		
+		// ++++++	
+		//   VEC3init(&wp, 1., 1., 1.);
+		//   MAT3eval(&wr, &From, &wp);
+		// ++++++
+
+	   
        // invert dest
 
+	   
        if (MAT3inverse(&To, &ToInv) < 0)
                         return FALSE;
 
        // Multiply
-
-        MAT3per(&Transfer, &ToInv, &From);
+        MAT3per(&Transfer, &ToInv, &From); 
+	
+		
+		// +++++
+		// MAT3eval(&wf, &ToInv, &wr);
+		// MAT3eval(&wf, &Transfer, &wp);
+		// ++++++
 
         // Ok, compute gamma trans
 
@@ -583,7 +601,7 @@ void TakeConversionRoutines(_LPcmsTRANSFORM p)
                              &IlluminantOut,
 
                  &p->Stage1,
-                 &p->a1, &p->b1);
+                 &p->m1, &p->of1);
 
        }
        else
@@ -606,7 +624,7 @@ void TakeConversionRoutines(_LPcmsTRANSFORM p)
                              &IlluminantProof,
 
                  &p->Stage1,
-                 &p->a1, &p->b1);
+                 &p->m1, &p->of1);
 
        cmsChooseCnvrt(p -> ProofIntent == INTENT_ABSOLUTE_COLORIMETRIC,
 
@@ -620,7 +638,7 @@ void TakeConversionRoutines(_LPcmsTRANSFORM p)
                              &WhitePointOut,
                              &IlluminantOut,
                  &p->Stage2,
-                 &p->a2, &p->b2);
+                 &p->m2, &p->of2);
 
 
        }
@@ -666,6 +684,30 @@ void PatchLUT(LPLUT Grid, WORD At[], WORD Value[],
 
 }
 
+// Replace endpoint of prelinearization curves, only
+// if these does exist
+
+	   
+static
+void PatchCurves(LPLUT Grid)
+{
+	unsigned int i;
+
+
+
+	if (Grid ->wFlags & LUT_HASTL1) {
+
+		for (i=0; i < Grid ->InputChan; i++) {
+
+		Grid ->L1[i][Grid->In16params.Domain] = 0xFFFF;
+		Grid ->L1[i][0] = 0;
+
+		_cmsSmoothEndpoints(Grid->L1[i], Grid->InputEntries);					
+		}
+	}		
+
+}
+
 
 
 static
@@ -706,6 +748,7 @@ void WhiteBlackCompensation(_LPcmsTRANSFORM p)
 
        PatchLUT(p -> DeviceLink, WhitePointIn, WhitePointOut, nOuts, nIns);
        PatchLUT(p -> DeviceLink, BlackPointIn, BlackPointOut, nOuts, nIns);
+	   PatchCurves(p -> DeviceLink);
 
 }
 
@@ -861,66 +904,71 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
               }
        }
 
+	   // Apr-15, 2002 - Too much profiles does have bogus content
+       // on preview tag, so I do compute it by my own.
+
        if (ProofingProfile)
        {
+           if (dwFlags & cmsFLAGS_SOFTPROOFING) {
 
-		   if (dwFlags & cmsFLAGS_SOFTPROOFING) {
+			  p -> Preview = _cmsComputeSoftProofLUT(ProofingProfile, ProofingIntent);                  
+			  if (p -> Preview == NULL) {
+				  
+				ProofTag = Preview[nIntent];
 
-              ProofTag = Preview[nIntent];
+				if (!cmsIsTag(ProofingProfile,  ProofTag)) {
 
-              if (!cmsIsTag(ProofingProfile,  ProofTag)) {
+                    ProofTag = Preview[0];
+                    if (!cmsIsTag(ProofingProfile,  ProofTag))
+                                    ProofTag = (icTagSignature)0;
+				}
 
-					ProofTag = Preview[0];
-					if (!cmsIsTag(ProofingProfile,  ProofTag))
-						            ProofTag = (icTagSignature)0;
-              }
-
-              if (ProofTag) {
+				if (ProofTag) {
 
                      p -> Preview = cmsReadICCLut(ProofingProfile, ProofTag);
                      ToTag  = PCS2Device[ProofingIntent];
-              }
-              else
+				}
+				else
                      {
                      p -> Preview = NULL;
                      ProofingProfile = NULL;
                      cmsSignalError(LCMS_ERRC_WARNING, "Sorry, the proof profile has not previewing capabilities");
                      }
+			  }
 
-			  // Preview available, link into render pipeline
+              // Preview available, link into render pipeline
 
-			  if (p -> Preview)
-						p -> Phase2 = GetPhase(ProofingProfile);
+              if (p -> Preview)
+                        p -> Phase2 = GetPhase(ProofingProfile);
 
-		   }
-			
-		   
-		   // Aug-31, 2001 - Too much profiles does have bogus content
-		   // on preview tag, so I do compute it by my own.
+           }
+            
+           
+           // Aug-31, 2001 - Too much profiles does have bogus content
+           // on gamut tag, so I do compute it by my own.
 
            if (dwFlags & cmsFLAGS_GAMUTCHECK) {
 
-					 
-				  	 p -> Gamut = _cmsComputeGamutLUT(ProofingProfile);
+                               
+                     p -> Gamut = _cmsComputeGamutLUT(ProofingProfile);                  
+                     if (p -> Gamut == NULL) {
 
-					 if (p -> Gamut == NULL) {
+                        // Profile goes only in one direction... try to see 
+                        // if profile has the tag, and use it, no matter it
+                         // could be bogus. This is the last chance!
 
-						// Profile goes only in one direction... try to see 
-						// if profile has the tag, and use it, no matter it
-						 // could be bogus. This is the last chance!
-
-						if (cmsIsTag(ProofingProfile, icSigGamutTag)) {
+                        if (cmsIsTag(ProofingProfile, icSigGamutTag)) {
 
                             p -> Gamut = cmsReadICCLut(ProofingProfile, icSigGamutTag);
-							}
-							else   {
-					 					
-							 // Nope, cannot be done.
+                            }
+                            else   {
+                                        
+                             // Nope, cannot be done.
 
                              cmsSignalError(LCMS_ERRC_WARNING, "Sorry, the proof profile has not gamut checking capabilities");
                              p -> Gamut = NULL;
-							}
-					 }
+                            }
+                     }
 
               }
        }
@@ -930,6 +978,7 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
 
        if (!cmsIsTag(OutputProfile,  ToTag))
        {
+			  	  
               ToTag = PCS2Device[0];
               if (!cmsIsTag(OutputProfile,  ToTag))
                             ToTag = (icTagSignature)0;
@@ -1029,7 +1078,7 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateProofingTransform(cmsHPROFILE InputProfile,
        {
 
                if (!_cmsPrecalculateDeviceLink(p,
-										(dwFlags & cmsFLAGS_HIGHRESPRECALC) ? 1 : 0)) {
+                                        (dwFlags & cmsFLAGS_HIGHRESPRECALC) ? 1 : 0)) {
 
                      cmsSignalError(LCMS_ERRC_ABORTED,
                                 "Cannot precalculate %d->%d channels transform!",
@@ -1163,16 +1212,16 @@ BOOL LCMSEXPORT cmsIsIntentSupported(cmsHPROFILE hProfile,
 static
 int MultiprofileSampler(register WORD In[], register WORD Out[], register LPVOID Cargo)
 {
-	cmsHTRANSFORM* Transforms = (cmsHTRANSFORM*) Cargo;
-	int i;
-	
-	cmsDoTransform(Transforms[0], In, Out, 1);
+    cmsHTRANSFORM* Transforms = (cmsHTRANSFORM*) Cargo;
+    int i;
+    
+    cmsDoTransform(Transforms[0], In, Out, 1);
 
-	for (i=1; Transforms[i]; i++)
+    for (i=1; Transforms[i]; i++)
         cmsDoTransform(Transforms[i], Out, Out, 1);
 
 
-	
+    
     return TRUE;
 }
 
@@ -1184,90 +1233,102 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateMultiprofileTransform(cmsHPROFILE hProfiles[],
                                                                 int Intent,
                                                                 DWORD dwFlags)
 {
-	cmsHTRANSFORM Transforms[257];
-	DWORD dwPrecalcFlags = (dwFlags|cmsFLAGS_NOTPRECALC);
-	DWORD RawFormat, RawIdentity;
-	cmsHPROFILE hLab, hXYZ, hProfile;	
-	icColorSpaceSignature FirstColorSpace;
-	LPLUT Grid;
-    int nGridPoints, ChannelsIn, ChannelsOut, i;	
-	_LPcmsTRANSFORM p;
-	icColorSpaceSignature ColorSpace;	
+    cmsHTRANSFORM Transforms[257];
+    DWORD dwPrecalcFlags = (dwFlags|cmsFLAGS_NOTPRECALC);
+    DWORD RawFormat, RawIdentity;
+    cmsHPROFILE hLab, hXYZ, hProfile;   
+    icColorSpaceSignature FirstColorSpace;
+    LPLUT Grid;
+    int nGridPoints, ChannelsIn, ChannelsOut, i;    
+    _LPcmsTRANSFORM p;
+    icColorSpaceSignature ColorSpace;
+    icProfileClassSignature Class;
 
-	if (nProfiles > 255) {
-		cmsSignalError(LCMS_ERRC_ABORTED, "What are you trying to do with more that 255 profiles?!?, of course aborted");
-		return NULL;
-	}
+    if (nProfiles > 255) {
+        cmsSignalError(LCMS_ERRC_ABORTED, "What are you trying to do with more that 255 profiles?!?, of course aborted");
+        return NULL;
+    }
 
 
-	// Creates a phantom transform for latter filling
+    // Creates a phantom transform for latter filling
     p = cmsCreateTransform(NULL, InputFormat, NULL, OutputFormat, Intent, cmsFLAGS_NULLTRANSFORM);
 
-	// If user wants null one, give it
-	if (dwFlags & cmsFLAGS_NULLTRANSFORM) return (cmsHPROFILE) p;
+    // If user wants null one, give it
+    if (dwFlags & cmsFLAGS_NULLTRANSFORM) return (cmsHPROFILE) p;
 
 
-	// We will need a 3DCLUT for device link
-	Grid =  cmsAllocLUT();
+    // We will need a 3DCLUT for device link
+    Grid =  cmsAllocLUT();
     if (!Grid) return NULL;
 
-	// This one is our PCS (Always Lab)
-	hLab  = cmsCreateLabProfile(NULL);
-	hXYZ  = cmsCreateXYZProfile();
+    // This one is our PCS (Always Lab)
+    hLab  = cmsCreateLabProfile(NULL);
+    hXYZ  = cmsCreateXYZProfile();
 
-	if (!hLab || !hXYZ) goto ErrorCleanup;
+    if (!hLab || !hXYZ) goto ErrorCleanup;
 
-	// Take some info....
-	FirstColorSpace = cmsGetColorSpace(hProfiles[0]);
-	ChannelsIn     = _cmsChannelsOf(FirstColorSpace);
-	ChannelsOut	   = 3;	// Will be updated on loop
+    // Take some info....
+    FirstColorSpace = cmsGetColorSpace(hProfiles[0]);
+    ChannelsIn     = _cmsChannelsOf(FirstColorSpace);
+    ChannelsOut    = 3; // Will be updated on loop
 
-	for (i=0; i < nProfiles; i++) {
+    for (i=0; i < nProfiles; i++) {
 
-		// Check colorspace, create transforms
-		hProfile	= hProfiles[i];
-		ColorSpace  = cmsGetColorSpace(hProfile);
-		ChannelsOut = _cmsChannelsOf(ColorSpace);
+        // Check colorspace, create transforms
+        hProfile    = hProfiles[i];
+        ColorSpace  = cmsGetColorSpace(hProfile);
+        ChannelsOut = _cmsChannelsOf(ColorSpace);
 
-		// Format: WORDs of current # of channels
-		RawFormat	 = BYTES_SH(2)|CHANNELS_SH(ChannelsOut);
-		RawIdentity  = BYTES_SH(2)|CHANNELS_SH(3);
+        // Format: WORDs of current # of channels
+        RawFormat    = BYTES_SH(2)|CHANNELS_SH(ChannelsOut);
+        RawIdentity  = BYTES_SH(2)|CHANNELS_SH(3);
 
-		if (ColorSpace == FirstColorSpace) {
+        if (ColorSpace == FirstColorSpace) {
 
-			FirstColorSpace = cmsGetPCS(hProfile);
-			Transforms[i]  = cmsCreateTransform(hProfile, RawFormat, 
-									(FirstColorSpace == icSigLabData ? hLab : hXYZ),
-									RawIdentity, Intent, dwPrecalcFlags);			
-		}
-		else  // Can come from pcs?
-		if (FirstColorSpace == icSigXYZData) {
+            FirstColorSpace = cmsGetPCS(hProfile);
+            Class           = cmsGetDeviceClass(hProfile);
 
-			Transforms[i] = cmsCreateTransform(hXYZ, RawIdentity, hProfile, RawFormat, Intent, dwPrecalcFlags);
-			FirstColorSpace = ColorSpace;
-			
-		}
-		else
-		if (FirstColorSpace == icSigLabData) {
+            if (Class == icSigLinkClass) {
 
-			Transforms[i] = cmsCreateTransform(hLab, RawIdentity, hProfile, RawFormat, Intent, dwPrecalcFlags);
-			FirstColorSpace = ColorSpace;
-		} 
-		else {
-				cmsSignalError(LCMS_ERRC_ABORTED, "cmsCreateMultiprofileTransform: ColorSpace mismatch");
-				goto ErrorCleanup;
-		}
+                    Transforms[i]  = cmsCreateTransform(hProfile, RawFormat, 
+                                    NULL, RawIdentity, Intent, dwPrecalcFlags);         
+            }
 
-	}
+            else {
 
-	// Mark end 
-	Transforms[i] = NULL;
+             Transforms[i]  = cmsCreateTransform(hProfile, RawFormat, 
+                                    (FirstColorSpace == icSigLabData ? hLab : hXYZ),
+                                    RawIdentity, Intent, dwPrecalcFlags);           
+            }
+        }
+        else  // Can come from pcs?
+        if (FirstColorSpace == icSigXYZData) {
 
-	nGridPoints = 48; // This always comes with 48 points
+            Transforms[i] = cmsCreateTransform(hXYZ, RawIdentity, hProfile, RawFormat, Intent, dwPrecalcFlags);
+            FirstColorSpace = ColorSpace;
+            
+        }
+        else
+        if (FirstColorSpace == icSigLabData) {
+
+            Transforms[i] = cmsCreateTransform(hLab, RawIdentity, hProfile, RawFormat, Intent, dwPrecalcFlags);
+            FirstColorSpace = ColorSpace;
+        } 
+        else {
+                cmsSignalError(LCMS_ERRC_ABORTED, "cmsCreateMultiprofileTransform: ColorSpace mismatch");
+                goto ErrorCleanup;
+        }
+
+    }
+
+    // Mark end 
+    Transforms[i] = NULL;
+
+    nGridPoints = 48; // This always comes with 48 points
     Grid = cmsAlloc3DGrid(Grid, nGridPoints, ChannelsIn, ChannelsOut);
 
     // Compute device link on 16-bit basis
-	   	   	    
+                
     if (!cmsSample3DGrid(Grid, MultiprofileSampler, (LPVOID) Transforms, Grid -> wFlags)) {
 
                 cmsFreeLUT(Grid);
@@ -1275,22 +1336,22 @@ cmsHTRANSFORM LCMSEXPORT cmsCreateMultiprofileTransform(cmsHPROFILE hProfiles[],
     }
 
     // All ok, store the newly created LUT
-	
+    
     p -> DeviceLink = Grid;
-	p -> xform		= PrecalculatedXFORM;
+    p -> xform      = PrecalculatedXFORM;
 
-	for (i=nProfiles-1; i > 0; --i)
-		cmsDeleteTransform(Transforms[i]);
+    for (i=nProfiles-1; i >= 0; --i)
+        cmsDeleteTransform(Transforms[i]);
 
 
-	if (hLab) cmsCloseProfile(hLab);
-	if (hXYZ) cmsCloseProfile(hXYZ);
-	return (cmsHTRANSFORM) p;
+    if (hLab) cmsCloseProfile(hLab);
+    if (hXYZ) cmsCloseProfile(hXYZ);
+    return (cmsHTRANSFORM) p;
 
 
 ErrorCleanup:
 
-	if (hLab) cmsCloseProfile(hLab);
-	if (hXYZ) cmsCloseProfile(hXYZ);
-	return NULL;
+    if (hLab) cmsCloseProfile(hLab);
+    if (hXYZ) cmsCloseProfile(hXYZ);
+    return NULL;
 }
