@@ -48,10 +48,15 @@
 typedef struct {BYTE r, g, b, a;}   Scanline_rgb1;
 typedef struct {WORD r, g, b, a;}   Scanline_rgb2;
 
+// Print a dot for gauging
 
+static 
+void Dot(void)
+{
+    fprintf(stdout, "."); fflush(stdout);
+}
 
-#ifndef LCMS_DLL
-
+// #ifndef LCMS_DLL
 
 // Are we little or big endian?  From Harbison&Steele.  
 
@@ -70,19 +75,82 @@ int CheckEndianess(void)
    BigEndian = (u.c[sizeof (long) - 1] == 1);
 
 #ifdef USE_BIG_ENDIAN
-		IsOk = BigEndian;
+        IsOk = BigEndian;
 #else
-		IsOk = !BigEndian;
+        IsOk = !BigEndian;
 #endif
 
-		if (!IsOk) {
+        if (!IsOk) {
 
-			printf("\nOOOPPSS! You have USE_BIG_ENDIAN toggle misconfigured!\n\n");
-			printf("Please, edit lcms.h and %s the USE_BIG_ENDIAN toggle.\n", BigEndian? "uncomment" : "comment");
-			return 0;
-		}
+            printf("\nOOOPPSS! You have USE_BIG_ENDIAN toggle misconfigured!\n\n");
+            printf("Please, edit lcms.h and %s the USE_BIG_ENDIAN toggle.\n", BigEndian? "uncomment" : "comment");
+            return 0;
+        }
 
+        return 1;
+
+}
+
+
+static
+int CheckSwab(void)
+{
+	unsigned char Test[] = { 1, 2, 3, 4, 5, 6};
+
+#ifdef USE_CUSTOM_SWAB
 		return 1;
+#endif
+
+	swab(Test, Test, 6);
+
+	if (strncmp(Test, "\x2\x1\x4\x3\x6\x5", 6) != 0)
+	{
+            printf("\nOOOPPSS! swab() does not work as expected in your machine!\n\n");
+            printf("Please, edit lcms.h and uncomment the USE_CUSTOM_SWAB toggle.\n");
+            return 0;
+
+	}
+	return 1;
+}
+
+typedef struct _Stats {
+                      double n, x, y, x2, y2, xy;
+                      double Peak;
+                      } STATS, FAR* LPSTATS;
+
+static void ClearStats(LPSTATS p)
+{       
+       p -> n = p -> x = p -> y = p -> x2 = p -> y2 = p -> xy 
+       = p -> Peak = 0.0;
+}
+
+static double Std(LPSTATS p)
+{
+       return sqrt((p->n*p->x2 - p->x * p->x) / (p->n*(p->n-1)));
+}
+
+
+
+static
+void PrintStatistics(clock_t atime, LPSTATS Stats)
+{
+
+       clock_t diff;
+       double a;
+
+       diff = clock() - atime;
+       a = (double) diff / CLK_TCK;
+
+       // These are statistics of 16 bit, so divide
+       // by 257 to get dE relative to 8 bits
+
+       printf("\ndE: mean=%g, SD=%g, max=%g ",
+                     (Stats->x / Stats -> n) / 257.,
+                     (Std(Stats)) / 257.,
+                     Stats -> Peak / 257.);
+
+       if (atime > 0)
+            printf("[%d tics, %g sec.]", (int) diff, a);
 
 }
 
@@ -148,6 +216,82 @@ int TestFixedScaling(void)
        return 1;
 }
 
+// Curve joining test. Joining two high-gamma of 3.0 curves should
+// give something like linear
+
+static
+int TestJointCurves(void)
+{
+    LPGAMMATABLE Forward, Reverse, Result;
+    BOOL rc;
+
+    printf("Testing curves join ...");
+
+    Forward = cmsBuildGamma(256, 3.0);
+    Reverse = cmsBuildGamma(256, 3.0);
+
+    Result = cmsJoinGammaEx(Forward, Reverse, 256);
+
+    cmsFreeGamma(Forward); cmsFreeGamma(Reverse); 
+
+    rc = cmsIsLinear(Result->GammaTable, Result ->nEntries);
+    cmsFreeGamma(Result); 
+
+    if (!rc) {
+        printf("failed!\n");
+        return 0;
+    }
+    else {
+        printf("pass.\n");
+        return 1;
+    }
+    
+}
+
+
+// Check reversing of gamma curves
+
+#define NPOINTS     1024
+
+static
+int TestReversingOfCurves(void)
+{
+    LPGAMMATABLE Gamma, Reverse, Computed;
+    int i;
+    double dE;
+    STATS Stats;
+
+
+    printf("Testing reversing of curves ...");
+    ClearStats(&Stats);
+
+
+    Gamma   = cmsBuildGamma(NPOINTS, 3.0);
+    Reverse = cmsBuildGamma(NPOINTS, 1.0/3.0);
+
+    Computed = cmsReverseGamma(NPOINTS, Gamma);
+    
+    for (i=0; i < NPOINTS; i++) {
+
+            dE = fabs(Reverse->GammaTable[i] - Computed->GammaTable[i]);
+
+            Stats.x += dE;                                   
+            Stats.x2 += (dE * dE);
+            Stats.n += 1.0;
+            if (dE > Stats.Peak) 
+                Stats.Peak = dE;
+
+            if (dE > 0x1100) {
+                printf("Coarse error! %x", (int) dE);
+                return 0;
+            }
+                                   
+    }
+
+    PrintStatistics(0, &Stats); 
+    printf(" pass.\n");
+    return 1;
+}
 
 // Linear interpolation test. Here I check the cmsLinearInterpLUT16
 // Tables are supposed to be monotonic, but the algorithm works on
@@ -509,7 +653,7 @@ int Test3D(void)
                                               0,   .5,  .25,    // B=1,G=1,R=0
 
                                               1,    0,    0,    // B=0,G=0,R=1
-                                              1,    0,  .25,	 // B=1,G=0,R=1
+                                              1,    0,  .25,     // B=1,G=0,R=1
 
                                               1,    .5,   0,    // B=0,G=1,R=1
                                               1,    .5,  .25    // B=1,G=1,R=1
@@ -525,7 +669,7 @@ int Test3D(void)
 
       // 2.- In this LUT, allocate a 3D grid of 2 points, from 3 components (RGB)
       //     to 3 components. First 3 is input dimension, last 3 is output one.
-      //		 2 is number of grid points.
+      //         2 is number of grid points.
 
       MyLut = cmsAlloc3DGrid(MyLut, 2, 3, 3);
 
@@ -537,10 +681,10 @@ int Test3D(void)
 
       for (i= 0; i < 3; i++)
        for (r = 0; r < 2; r++)
-   	     for (g = 0; g < 2; g++)
+         for (g = 0; g < 2; g++)
           for (b = 0; b < 2; b++) {
 
-         	WORD a =  (WORD) floor(*SampleTablePtr++ * 65535. + .5);
+            WORD a =  (WORD) floor(*SampleTablePtr++ * 65535. + .5);
 
             *Table++ = a;
         }
@@ -548,22 +692,22 @@ int Test3D(void)
 
    // The sample table gives
    //
-   // 		 r = input,
-   //		    g = input divided by 2
-   //			 b = input divided by 4
+   //        r = input,
+   //           g = input divided by 2
+   //            b = input divided by 4
    //
    // So, I should obtain on output r, g/2, g/4
 
 
    for (i=0; i < 0xffff; i++) {
 
-   	   In[0] = In[1] = In[2] = (WORD) i;
+       In[0] = In[1] = In[2] = (WORD) i;
 
-	   cmsEvalLUT(MyLut, In, Out);
+       cmsEvalLUT(MyLut, In, Out);
 
       // Check results, I will tolerate error <= 1  for rounding
 
-	   // printf("%x, %x\n", In[0], Out[0]);
+       // printf("%x, %x\n", In[0], Out[0]);
 
        if (!IsGood("Channel 1", Out[0], In[0])) return 0;
        if (!IsGood("Channel 2", Out[1], (WORD) ((double) In[1] / 2))) return 0;
@@ -698,7 +842,7 @@ void AdaptationMatrixTest(void)
        PrintMatrix(&TosRGB);
 }
 #endif
-#endif
+// #endif
 
 
 
@@ -716,46 +860,6 @@ double VecDist(Scanline_rgb2 *bin, Scanline_rgb2 *bout)
 
 
 
-typedef struct _Stats {
-                      double n, x, y, x2, y2, xy;
-                      double Peak;
-                      } STATS, FAR* LPSTATS;
-
-static void ClearStats(LPSTATS p)
-{       
-	   p -> n = p -> x = p -> y = p -> x2 = p -> y2 = p -> xy 
-       = p -> Peak = 0.0;
-}
-
-static double Std(LPSTATS p)
-{
-       return sqrt((p->n*p->x2 - p->x * p->x) / (p->n*(p->n-1)));
-}
-
-
-
-static
-void PrintStatistics(clock_t atime, LPSTATS Stats)
-{
-
-       clock_t diff;
-       double a;
-
-       diff = clock() - atime;
-       a = (double) diff / CLK_TCK;
-
-       // These are statistics of 16 bit, so divide
-       // by 257 to get dE relative to 8 bits
-
-       printf("\ndE: mean=%g, SD=%g, max=%g ",
-                     (Stats->x / Stats -> n) / 257.,
-                     (Std(Stats)) / 257.,
-                     Stats -> Peak / 257.);
-
-	   if (atime > 0)
-			printf("[%d tics, %g sec.]", (int) diff, a);
-
-}
 
 
 
@@ -787,7 +891,7 @@ int TestFullSpectrum(cmsHTRANSFORM xform, int nRedInterv)
        {
               // printf("\r%02x:", r);
 
-			  printf(".");
+              Dot();
               for (g=0; g < 256; g++)
                      {
 
@@ -853,12 +957,11 @@ int TestInducedError(DWORD Type)
        xform = cmsCreateTransform(In,  Type,
                                   Out, Type,
                                   INTENT_RELATIVE_COLORIMETRIC, 0);
-       
-       assert(xform);
+              
 
        nMaxError = TestFullSpectrum(xform, 31);
 
-	   printf("\n");
+       printf("\n");
 
        cmsDeleteTransform(xform);
        cmsCloseProfile(In);
@@ -893,7 +996,8 @@ double Convertab(WORD v)
 #define BASE  255
 
 static
-int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2, int nRedInterv, int lWithStats)
+int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2, 
+                      int nRedInterv, int lWithStats, BOOL lIsLab)
 {
        int r, g, b;
        double err;
@@ -908,14 +1012,14 @@ int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2, int nRedInterv
 
 
        ClearStats(&Stats);
-	   Stats.x = 0.0; Stats.n = 0.0; // GCC BUG HERE!!!!
+       Stats.x = 0.0; Stats.n = 0.0; // GCC BUG HERE!!!!
 
 
        for (r=0; r < BASE; r+= nRedInterv)
        {
               // printf("\r%02x:", r);
 
-			  printf(".");
+              Dot();
               for (g=0; g < BASE; g++)
                      {
                             // I will test random LSB
@@ -947,12 +1051,20 @@ int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2, int nRedInterv
                                    if (err > 0x1000L)
                                    {
 
-
+                                       if (lIsLab) {
                                           printf("Coarse error: In=(%x,%x,%x) Out1=(%g,%g,%g) Out2=(%g,%g,%g)\n",
                                                         bin[b].r, bin[b].g, bin[b].b,
                                                         ConvertL(bout1[b].r), Convertab(bout1[b].g), Convertab(bout1[b].b),
                                                         ConvertL(bout2[b].r), Convertab(bout2[b].g), Convertab(bout2[b].b));
-                                          return 0;
+                                       }
+                                       else
+                                       {
+                                        printf("Coarse error: In=(%x,%x,%x) Out1=(%x,%x,%x) Out2=(%x,%x,%x)\n",
+                                                        bin[b].r, bin[b].g, bin[b].b,
+                                                        bout1[b].r, bout1[b].g, bout1[b].b,
+                                                        bout2[b].r, bout2[b].g, bout2[b].b);
+                                       }
+                                       return 0;
 
                                    }
 
@@ -973,11 +1085,11 @@ int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2, int nRedInterv
        }
 
 
-	   if (lWithStats) {
+       if (lWithStats) {
 
-		PrintStatistics(0, &Stats);
-		printf(" pass.\n");
-	   }
+        PrintStatistics(0, &Stats);
+        printf(" pass.\n");
+       }
 
        if (OutOfGamut > 0)
                 printf("Out of encodeable representation=%d\n\n", OutOfGamut);
@@ -991,10 +1103,132 @@ int CompareTransforms(cmsHTRANSFORM xform1, cmsHTRANSFORM xform2, int nRedInterv
 }
 
 
+
+static
+BOOL CheckXYZ(LPcmsCIEXYZ Check, double X, double Y, double Z)
+{
+    return ((fabs(Check->X - X) < 0.001) && 
+            (fabs(Check->Y - Y) < 0.001) &&
+            (fabs(Check->Z - Z) < 0.001));
+}
+
+
+static
+LPGAMMATABLE Build_sRGBGamma(void)
+{
+    double Parameters[5];
+
+    Parameters[0] = 2.4;
+    Parameters[1] = 1. / 1.055;
+    Parameters[2] = 0.055 / 1.055;
+    Parameters[3] = 1. / 12.92;
+    Parameters[4] = 0.04045;    // d
+
+    return cmsBuildParametricGamma(1024, 3, Parameters);
+}
+
+static
+BOOL Check_sRGBGamma(LPGAMMATABLE Shape)
+{
+    LPGAMMATABLE sRGB = Build_sRGBGamma();
+    int i;
+
+    if (Shape ->nEntries != 1024) { 
+        printf("because wrong sizes (%d != 1024), ", Shape -> nEntries); 
+        return 0;
+    }
+
+
+    for (i=0; i < Shape -> nEntries; i++) {
+        double nErr = Shape ->GammaTable[i] - sRGB ->GammaTable[i];
+
+        if (fabs(nErr) > 1.0) {
+                    int j;
+                    printf("because %x != %x on index %d\n", Shape ->GammaTable[i], sRGB ->GammaTable[i], i);
+                    printf("table dump follows:\n");
+                    for (j=0; j < 10; j++)
+                            printf("%d) %X\n", j, Shape ->GammaTable[j]);
+                    printf("\nso, ");
+
+                    return 0;
+        }
+    }
+
+    cmsFreeGamma(sRGB);
+    return 1;
+}
+
+
+static
+int GetInfoTest(void)
+{
+    cmsHPROFILE hProfile;
+    cmsCIEXYZ WhitePoint;
+    cmsCIEXYZTRIPLE Primaries;
+    const char* Product;
+    LPGAMMATABLE Shapes[3];
+
+
+    printf("Testing profile decoding (sRGB)");
+    hProfile = cmsOpenProfileFromFile("sRGB Color Space Profile.icm", "rb");
+    cmsTakeMediaWhitePoint(&WhitePoint, hProfile);
+
+    Dot();
+    if (!CheckXYZ(&WhitePoint, 0.95045, 1.0, 1.08905)) {
+        printf("White point read failed!\n");
+        return 0;
+    }
+
+    Dot();
+    cmsTakeColorants(&Primaries, hProfile);
+
+    if (!CheckXYZ(&Primaries.Red, 0.43607, 0.22249, 0.01392)) {
+        printf("Red colorant failed!\n");
+        return 0;
+    }
+
+    if (!CheckXYZ(&Primaries.Green, 0.38515,0.71617, 0.09708)) {
+        printf("Green colorant failed!\n");
+        return 0;
+    }
+
+    if (!CheckXYZ(&Primaries.Blue, 0.14307, 0.06061, 0.71410)) {
+        printf("Blue colorant failed!\n");
+        return 0;
+    }
+
+    Dot();
+    Product = cmsTakeProductName(hProfile);
+    if (strcmp(Product, "IEC 61966-2.1 Default RGB colour space - sRGB") != 0) {        
+    printf("Product name mismatch!\n");
+    }
+
+    Dot();
+    Shapes[0] = cmsReadICCGamma(hProfile, icSigRedTRCTag);
+    Shapes[1] = cmsReadICCGamma(hProfile, icSigGreenTRCTag);
+    Shapes[2] = cmsReadICCGamma(hProfile, icSigBlueTRCTag);
+
+    if (!Check_sRGBGamma(Shapes[0]) ||
+        !Check_sRGBGamma(Shapes[1]) ||
+        !Check_sRGBGamma(Shapes[2])) {
+            printf("Gamma curves mismatch!\n");
+            return 0;
+    }
+
+
+    cmsFreeGammaTriple(Shapes);
+    
+
+    Dot();
+    cmsCloseProfile(hProfile);
+    printf("pass.\n");
+    return 1;
+
+}
 static
 int Test_sRGB(void)
 {
-	   cmsHPROFILE In1, In2, Out1, Out2;
+       cmsHPROFILE In1, In2, Out1, Out2;
        cmsHTRANSFORM xform1, xform2;
        int nMaxErr;
 
@@ -1009,7 +1243,7 @@ int Test_sRGB(void)
        xform1 = cmsCreateTransform(In1, TYPE_RGBA_16, Out1, TYPE_XYZA_16, 0, cmsFLAGS_NOTPRECALC);
        xform2 = cmsCreateTransform(In2, TYPE_RGBA_16, Out2, TYPE_XYZA_16, 0, cmsFLAGS_NOTPRECALC);
 
-       nMaxErr = CompareTransforms(xform1, xform2, 31, TRUE);
+       nMaxErr = CompareTransforms(xform1, xform2, 31, TRUE, FALSE);
 
        cmsDeleteTransform(xform1);
        cmsCloseProfile(In1);
@@ -1053,9 +1287,9 @@ int RealProfilesTest(void)
        xform1 = cmsCreateTransform(In1, TYPE_RGBA_16, Out1, TYPE_XYZA_16, 0, cmsFLAGS_NOTPRECALC|cmsFLAGS_MATRIXINPUT);
        xform2 = cmsCreateTransform(In2, TYPE_RGBA_16, Out2, TYPE_XYZA_16, 0, cmsFLAGS_NOTPRECALC);
 
-       nMaxErr = CompareTransforms(xform1, xform2, 31, FALSE);
+       nMaxErr = CompareTransforms(xform1, xform2, 31, FALSE, FALSE);
 
-	   printf("pass\n");
+       printf("pass\n");
 
        cmsDeleteTransform(xform1);
        cmsCloseProfile(In1);
@@ -1094,7 +1328,7 @@ int TestPreview(void)
        cmsCloseProfile(In);
        cmsCloseProfile(Out);
        cmsCloseProfile(Proof);
-	   printf("\n");
+       printf("\n");
 
        return nMaxErr;
 }
@@ -1106,168 +1340,204 @@ static
 int TestMultiprofile(void)
 {
 
-	cmsHPROFILE hsRGB, hXYZ, hLab;
-	cmsHTRANSFORM hXForm;
-	cmsHPROFILE Profiles[10];
-	int nMaxErr;
+    cmsHPROFILE hsRGB, hXYZ, hLab;
+    cmsHTRANSFORM hXForm;
+    cmsHPROFILE Profiles[10];
+    int nMaxErr;
 
-	hsRGB = cmsCreate_sRGBProfile();
-	hLab = cmsCreateLabProfile(NULL);
-	hXYZ = cmsCreateXYZProfile();
+    hsRGB = cmsCreate_sRGBProfile();
+    hLab = cmsCreateLabProfile(NULL);
+    hXYZ = cmsCreateXYZProfile();
 
-	Profiles[0] = hsRGB;
-	Profiles[1] = hLab;
-	Profiles[2] = hsRGB;		
-	Profiles[3] = hsRGB;		
+    Profiles[0] = hsRGB;
+    Profiles[1] = hLab;
+    Profiles[2] = hsRGB;        
+    Profiles[3] = hsRGB;        
 
-	Profiles[4] = hLab;
-	Profiles[5] = hXYZ;
-	Profiles[6] = hsRGB;
+    Profiles[4] = hLab;
+    Profiles[5] = hXYZ;
+    Profiles[6] = hsRGB;
 
-	hXForm = cmsCreateMultiprofileTransform(Profiles, 7, TYPE_RGBA_16, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, 0);
-	
-	printf("Checking multiprofile transforms (6 profiles):");
+    hXForm = cmsCreateMultiprofileTransform(Profiles, 7, TYPE_RGBA_16, TYPE_RGBA_16, INTENT_RELATIVE_COLORIMETRIC, 0);
+    
+    printf("Checking multiprofile transforms (6 profiles):");
 
-	nMaxErr = TestFullSpectrum(hXForm, 31);
+    nMaxErr = TestFullSpectrum(hXForm, 31);
 
-	cmsDeleteTransform(hXForm);
+    cmsDeleteTransform(hXForm);
     cmsCloseProfile(hsRGB);
     cmsCloseProfile(hXYZ);
     cmsCloseProfile(hLab);
 
-	if (!nMaxErr) printf("pass.\n");
+    if (!nMaxErr) printf("pass.\n");
 
-	return nMaxErr;
+    return nMaxErr;
 
 
 }
 
+
+static
+void CheckPlanar(void)
+{
+    cmsHTRANSFORM xform;
+    cmsHPROFILE hsRGB;
+    int i;
+    BYTE Out[12];
+    BYTE Bmp[] = { 0x00, 0x10, 0x20, 0x30,   // R Plane  
+                   0x00, 0x10, 0x20, 0x30,   // G Plane
+                   0x00, 0x10, 0x20, 0x30 }; // B Plane
+    
+
+    
+            hsRGB = cmsCreate_sRGBProfile();
+            xform = cmsCreateTransform(hsRGB, TYPE_RGB_8_PLANAR, 
+                                       hsRGB, TYPE_RGB_8_PLANAR, 
+                                       INTENT_PERCEPTUAL, cmsFLAGS_NOTPRECALC);
+
+            cmsDoTransform(xform, Bmp, Out, 4);
+
+            for (i=0; i < 12; i += 3) {
+                    printf("RGB=(%x, %x, %x)\n", Out[i+0], Out[i+1], Out[i+2]);
+            }
+
+        cmsDeleteTransform(xform);
+        cmsCloseProfile(hsRGB);
+}
+
+
 #ifndef NON_WINDOWS
 
 static
 void CompareWithICM(void)
 {
 
-	HTRANSFORM hICMxform;
-	HPROFILE   hICMProfileFrom, hICMProfileTo;
-	LOGCOLORSPACE LogColorSpace;
-	COLOR In, Out;
-	int r, g, b;
-	PROFILE Profile;
-	clock_t atime;
-	double seconds, diff;
-	cmsHPROFILE hlcmsProfileIn, hlcmsProfileOut;
-	cmsHTRANSFORM hlcmsxform;
+    HTRANSFORM hICMxform;
+    HPROFILE   hICMProfileFrom, hICMProfileTo;
+    LOGCOLORSPACE LogColorSpace;
+    COLOR In, Out;
+    int r, g, b;
+    PROFILE Profile;
+    clock_t atime;
+    double seconds, diff;
+    cmsHPROFILE hlcmsProfileIn, hlcmsProfileOut;
+    cmsHTRANSFORM hlcmsxform;
 
 
-	printf("\n\nComparative with MS-Windows ICM:\n");
-	
+    printf("\n\nComparative with MS-Windows ICM:\n");
+    
 
-	Profile.dwType = PROFILE_FILENAME;
-	Profile.pProfileData = "sRGBSpac.ICM";
+    Profile.dwType = PROFILE_FILENAME;
+    Profile.pProfileData = "sRGBSpac.ICM";
     Profile.cbDataSize   = strlen("sRGBSpac.ICM");
 
-	hICMProfileFrom = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
+    hICMProfileFrom = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
 
-	Profile.pProfileData = "sRGBSpac.ICM";
-	Profile.cbDataSize   = strlen("sRGBSpac.ICM");
+    Profile.pProfileData = "sRGBSpac.ICM";
+    Profile.cbDataSize   = strlen("sRGBSpac.ICM");
     hICMProfileTo   = OpenColorProfile(&Profile, PROFILE_READ, FILE_SHARE_READ, OPEN_EXISTING);
 
-	ZeroMemory(&LogColorSpace, sizeof(LOGCOLORSPACE));
+    ZeroMemory(&LogColorSpace, sizeof(LOGCOLORSPACE));
 
-	LogColorSpace.lcsSignature = LCS_SIGNATURE;
-	LogColorSpace.lcsVersion   = 0x400;
-	LogColorSpace.lcsCSType    = LCS_CALIBRATED_RGB;
-	strcpy(LogColorSpace.lcsFilename, "sRGBSpac.ICM");
+    LogColorSpace.lcsSignature = LCS_SIGNATURE;
+    LogColorSpace.lcsVersion   = 0x400;
+    LogColorSpace.lcsCSType    = LCS_CALIBRATED_RGB;
+    strcpy(LogColorSpace.lcsFilename, "sRGBSpac.ICM");
 
-	hICMxform = CreateColorTransform(&LogColorSpace, hICMProfileTo, NULL, BEST_MODE);
+    hICMxform = CreateColorTransform(&LogColorSpace, hICMProfileTo, NULL, BEST_MODE);
 
-	printf("Windows ICM is transforming full spectrum...");
+    printf("Windows ICM is transforming full spectrum...");
 
-	atime = clock();
+    atime = clock();
 
-	for (r=0; r < 255; r++)
-		for (g=0; g < 255; g++)
-			for (b=0; b < 255; b++) {
+    for (r=0; r < 255; r++)
+        for (g=0; g < 255; g++)
+            for (b=0; b < 255; b++) {
 
-		In.rgb.red   = (r << 8) | r;
-		In.rgb.green = (g << 8) | g;
-		In.rgb.blue  = (b << 8) | b;
+        In.rgb.red   = (r << 8) | r;
+        In.rgb.green = (g << 8) | g;
+        In.rgb.blue  = (b << 8) | b;
 
-		TranslateColors( hICMxform, &In, 1, COLOR_RGB, &Out, COLOR_RGB);
-	}
+        TranslateColors( hICMxform, &In, 1, COLOR_RGB, &Out, COLOR_RGB);
+    }
 
-	diff = clock() - atime;
+    diff = clock() - atime;
     seconds = (double) diff / CLK_TCK;
 
 
-	printf("done. [%d tics, %g sec.]\n", (int) diff, seconds);
+    printf("done. [%d tics, %g sec.]\n", (int) diff, seconds);
   
-	CloseColorProfile(hICMProfileFrom);
-	CloseColorProfile(hICMProfileTo);
-	DeleteColorTransform(hICMxform);
+    CloseColorProfile(hICMProfileFrom);
+    CloseColorProfile(hICMProfileTo);
+    DeleteColorTransform(hICMxform);
 
-	hlcmsProfileIn  = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
-	hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
+    hlcmsProfileIn  = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
+    hlcmsProfileOut = cmsOpenProfileFromFile("sRGBSpac.ICM", "r");
 
-	hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
+    hlcmsxform  = cmsCreateTransform(hlcmsProfileIn, TYPE_RGB_16, hlcmsProfileOut, TYPE_RGB_16, INTENT_PERCEPTUAL, 0);
 
     printf("lcms is transforming full spectrum...");
 
-	atime = clock();
+    atime = clock();
 
-	for (r=0; r < 255; r++)
-		for (g=0; g < 255; g++)
-			for (b=0; b < 255; b++) {
+    for (r=0; r < 255; r++)
+        for (g=0; g < 255; g++)
+            for (b=0; b < 255; b++) {
 
-		In.rgb.red   = (r << 8) | r;
-		In.rgb.green = (g << 8) | g;
-		In.rgb.blue  = (b << 8) | b;
+        In.rgb.red   = (r << 8) | r;
+        In.rgb.green = (g << 8) | g;
+        In.rgb.blue  = (b << 8) | b;
 
-		cmsDoTransform(hlcmsxform, &In.rgb, &Out.rgb, 1);
-	}
+        cmsDoTransform(hlcmsxform, &In.rgb, &Out.rgb, 1);
+    }
 
-	diff = clock() - atime;
+    diff = clock() - atime;
     seconds = (double) diff / CLK_TCK;
 
-	printf("done. [%d tics, %g sec.]\n", (int) diff, seconds);
+    printf("done. [%d tics, %g sec.]\n", (int) diff, seconds);
 
-	cmsDeleteTransform(hlcmsxform);
-	cmsCloseProfile(hlcmsProfileIn);
-	cmsCloseProfile(hlcmsProfileOut);
+    cmsDeleteTransform(hlcmsxform);
+    cmsCloseProfile(hlcmsProfileIn);
+    cmsCloseProfile(hlcmsProfileOut);
 
 }
 
 #endif
 
+
+
 int main(int argc, char *argv[])
 {
        int lExhaustive = 0;
-
-
-       printf("little cms testbed. Ver 1.08 [build %s %s]\n\n", __DATE__, __TIME__);
-	  
+       
+       printf("little cms testbed. Ver 1.09 [build %s %s]\n\n", __DATE__, __TIME__);
+      
 #ifndef LCMS_DLL
 
-	   if (!CheckEndianess()) return 1;
+       if (!CheckEndianess()) return 1;
+	   if (!CheckSwab()) return 1;
 
        TestFixedPoint();
-       if (!TestFixedScaling()) return 1;
+       if (!TestFixedScaling()) return 1;     
+       if (!TestJointCurves()) return 1;
+       if (!TestReversingOfCurves()) return 1;
        if (!TestLinearInterpolation(lExhaustive)) return 1;
        if (!TestReverseLinearInterpolation()) return 1;
        if (!Test3D()) return 1;
        if (!TestMatrixCreation()) return 1;
+       if (!GetInfoTest()) return 1;
+
 #endif
 
-	   if (!Test_sRGB()) return 1;
+       if (!Test_sRGB()) return 1;
        if (!RealProfilesTest()) return 1;
        if (!TestInducedError(TYPE_LABA_16)) return 1;
 
        if (!TestPreview()) return 1;
-	   if (!TestMultiprofile()) return 1;
+       if (!TestMultiprofile()) return 1;
 
 #ifndef NON_WINDOWS
-	   CompareWithICM();
+       CompareWithICM();
 #endif
 
        printf("\nSuccess.\n");
